/**************************************************************************//**
 * @file     fmc.c
 * @version  V1.00
 * $Revision: 1 $
 * $Date: 14/11/17 2:37p $ 
 * @brief    MINI51X series FMC driver source file
 *
 * @note
 * Copyright (C) 2014 Nuvoton Technology Corp. All rights reserved.
 *****************************************************************************/  

//* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include "Mini51XSeries.h"

/** @addtogroup MINI51X_Device_Driver MINI51X Device Driver
  @{
*/

/** @addtogroup MINI51X_FMC_Driver FMC Driver
  @{
*/


/** @addtogroup MINI51X_FMC_EXPORTED_FUNCTIONS FMC Exported Functions
  @{
*/
extern uint32_t g_apromSize, g_dataFlashAddr, g_dataFlashSize;

/**
  * @brief    Writes a word data to specified flash address.
  * @param[in]    u32Addr: destination address
  * @param[in]   u32Data: word data to be written
  */
int FMC_Write(uint32_t u32Addr, uint32_t u32Data)
{
#if(0)
    FMC->ISPCMD = FMC_ISPCMD_PROGRAM;
    FMC->ISPADDR = u32Addr;
    FMC->ISPDAT = u32Data;
    FMC->ISPTRG = FMC_ISPTRG_ISPGO_Msk; 
    while (FMC->ISPTRG & FMC_ISPTRG_ISPGO_Msk) ;
#endif
	
	unsigned int Reg;
    
    outp32(&FMC->ISPCMD, ISP_Program);
    outp32(&FMC->ISPADDR, u32Addr);
    outp32(&FMC->ISPDAT, u32Data);
    outp32(&FMC->ISPTRG, ISPGO); 
    
    __ISB();
    
    Reg = inp32(&FMC->ISPCTL);
    if (Reg & ISPFF)
    {
    	outp32(&FMC->ISPCTL, Reg);
        return -1;
    }
    return 0;
}


/**
  * @brief    Read a word from specified flash address. 
  * @param[in]    u32Addr: Flash word address. Must be a word aligned address.
  * @retval   The word data stored in the flash address "u32Addr".
  */
//uint32_t FMC_Read(uint32_t u32Addr)
int32_t FMC_Read(uint32_t u32Addr, unsigned int * data)
{ 
#if(0)
    FMC->ISPCMD = FMC_ISPCMD_READ;
    FMC->ISPADDR = u32Addr;
    FMC->ISPTRG = FMC_ISPTRG_ISPGO_Msk; 
    
    while (FMC->ISPTRG & FMC_ISPTRG_ISPGO_Msk) ;

    return FMC->ISPDAT;
#else
	
    unsigned int Reg;

    outp32(&FMC->ISPCMD, ISP_Read);
    outp32(&FMC->ISPADDR, u32Addr);
    outp32(&FMC->ISPDAT, 0x00000000);
	  outp32(&FMC->ISPTRG, ISPGO); 
    
    __ISB();

    Reg = inp32(&FMC->ISPCTL);
    if (Reg & ISPFF)
    {
    	outp32(&FMC->ISPCTL, Reg);
    	return -1;
    }
    
	*data = inp32(&FMC->ISPDAT);
    return 0;
#endif		
}


/**
  * @brief    Erase a page. The page size is 512 bytes. 
  * @param[in]    u32PageAddr: Flash page address. Must be a 512-byte aligned address.
  * @retval   0: Success
  * @retval   -1: Erase failed
  */
int32_t FMC_Erase(uint32_t u32PageAddr)
{
#if(0)
	  FMC->ISPCMD = FMC_ISPCMD_PAGE_ERASE;
    FMC->ISPADDR = u32PageAddr;
    FMC->ISPTRG = FMC_ISPTRG_ISPGO_Msk; 
    
    while (FMC->ISPTRG & FMC_ISPTRG_ISPGO_Msk) ;

    if (FMC->ISPCTL & FMC_ISPCTL_ISPFF_Msk)
    {
        FMC->ISPCTL |= FMC_ISPCTL_ISPFF_Msk;
        return -1;
    }
    return 0;
		
#else
    unsigned int Reg;
    
    outp32(&FMC->ISPCMD, ISP_PageErase);
    outp32(&FMC->ISPADDR, u32PageAddr);
    outp32(&FMC->ISPTRG, ISPGO); 

	__ISB();
    
    Reg = inp32(&FMC->ISPCTL);
    if (Reg & ISPFF)
    {
    	outp32(&FMC->ISPCTL, Reg);
    	return -1;
    }
    return 0;
#endif		
}


void ReadData(unsigned int addr_start, unsigned int addr_end, unsigned int* data)    // Read data from flash
{
    unsigned int rLoop;

    for ( rLoop = addr_start; rLoop < addr_end; rLoop += 4 ) 
    {     
		FMC_Read(rLoop, data);
		data++;
    }
    return;
}

void WriteData(unsigned int addr_start, unsigned int addr_end, unsigned int *data)  // Write data into flash
{
    unsigned int wLoop;
    
    for ( wLoop = addr_start; wLoop < addr_end; wLoop+=4 ) 
    {
        FMC_Write(wLoop, *data);
        data++;
    }
}


//bAprom == TRUE erase all aprom besides data flash
void EraseAP(BOOL bAprom, unsigned int addr_start, unsigned int addr_end)
{
    unsigned int eraseLoop;
    unsigned int erase_end;
    
    if(bAprom == TRUE)
    {
    	//GetDataFlashInfo(&erase_end, &eraseLoop);
    	erase_end = g_dataFlashAddr;
    	eraseLoop = FMC_APROM_BASE;
    }
    else
    {
    	eraseLoop = addr_start;
    	erase_end = addr_end;
    }
    for ( ; eraseLoop < erase_end; eraseLoop += FMC_FLASH_PAGE_SIZE )
    {
        FMC_Erase(eraseLoop);
    }           
    return;
}

void UpdateConfig(unsigned int *data, unsigned int *res)
{		
	
	  FMC->ISPCTL |= FMC_ISPCTL_CFGUEN_Msk;//enable config update
    FMC_Erase(Config0);
    //FMC_Write(Config0, 0xFFFFFF7F); // Disable Lock bit
    //FMC_Write(Config1, 0x0001F000);
    
   	FMC_Write(Config0, *data);
   	FMC_Write(Config1, *(data+1));
        
    if(res)
    {
    	FMC_Read(Config0, res);
    	FMC_Read(Config1, res+1);
    }
    
    FMC->ISPCTL &= ~(FMC_ISPCTL_CFGUEN_Msk);//disable config update

}

//Mini51 Data flash shared with APROM regardless APROM size
void GetDataFlashInfo(uint32_t *addr, uint32_t *size)
{
	uint32_t uData;
	
	*size = 0;
	

   	FMC_Read(Config0, &uData);
   	if((uData&0x01)==0)//DFEN enable
   	{
   		FMC_Read(Config1, &uData);
   		if((uData >= g_apromSize) || (uData < 0x200))//avoid config1 value from error
    		uData = g_apromSize;
    		
    	*addr = uData;
    	*size = g_apromSize - uData;
    }
    else//no data flash
    {
    	*addr = g_apromSize;
    	*size = 0;
    }
    
}

/*@}*/ /* end of group MINI51X_FMC_EXPORTED_FUNCTIONS */

/*@}*/ /* end of group MINI51X_FMC_Driver */

/*@}*/ /* end of group MINI51X_Device_Driver */

/*** (C) COPYRIGHT 2014 Nuvoton Technology Corp. ***/


