/****************************************************************************
 * @file     main.c
 * @version  V2.00
 * $Revision: 1 $
 * $Date: 14/11/17 2:37p $ 
 * @brief    Transmit and receive data from PC terminal through RS232 interface.
 *
 * @note
 * Copyright (C) 2014 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include <stdio.h>
#include "Mini51XSeries.h"
#include "FMC.h"
#include "gpio.h"

#define FW_VERSION		0x26


#define CMD_UPDATE_APROM	0x000000A0
#define CMD_UPDATE_CONFIG	0x000000A1
#define CMD_READ_CONFIG		0x000000A2
#define CMD_ERASE_ALL		  0x000000A3
#define CMD_SYNC_PACKNO		0x000000A4
#define CMD_GET_FWVER		  0x000000A6
#define CMD_RUN_APROM		  0x000000AB
#define CMD_RUN_LDROM		  0x000000AC
#define CMD_RESET			    0x000000AD
#define CMD_CONNECT			  0x000000AE
#define CMD_DISCONNECT		0x000000AF

#define CMD_GET_DEVICEID	0x000000B1

#define CMD_UPDATE_DATAFLASH 	0x000000C3
#define CMD_WRITE_CHECKSUM 	 	0x000000C9
#define CMD_GET_FLASHMODE 	 	0x000000CA

#define CMD_RESEND_PACKET       0x000000FF

#define	V6M_AIRCR_VECTKEY_DATA	0x05FA0000UL
#define V6M_AIRCR_SYSRESETREQ	  0x00000004UL

#define PACKET_SIZE	64

#define DISCONNECTED	0
#define CONNECTING		1
#define CONNECTED		2

static uint8_t volatile	bufhead;
static uint8_t volatile	g_connStatus;
__align(4) static uint8_t uart_rcvbuf[64];
__align(4) static uint8_t uart_sendbuf[64];
__align(4) static uint8_t aprom_buf[FMC_FLASH_PAGE_SIZE];
BOOL bUartDataReady;
BOOL bUpdateApromCmd;
uint32_t g_apromSize, g_dataFlashAddr, g_dataFlashSize;
volatile uint32_t g_pdid, g_timecnt;
static volatile int  g_enter_ISP;


void WDT_IRQHandler(void)
{
    //if(WDT_GET_TIMEOUT_INT_FLAG() == 1)
    if(WDT->CTL & WDT_CTL_IF_Msk) 
    {
        /* Clear WDT time-out interrupt flag */
        //WDT_CLEAR_TIMEOUT_INT_FLAG();
        WDT->CTL = (WDT->CTL & ~(WDT_CTL_RSTF_Msk | WDT_CTL_WKF_Msk)) | WDT_CTL_IF_Msk;
        bufhead = 0;

        WDT->CTL &= ~WDT_CTL_WDTEN_Msk;//_WDT_DISABLE_COUNTING();
       
    }		
}

/*
* send data for UART
*/
static __inline void PutString()
{
    int i;

    for(i = 0; i < PACKET_SIZE; i++)
    {
        while ((UART->FIFOSTS & UART_FIFOSTS_TXPTR_Msk)); 
        UART0->DAT = uart_sendbuf[i];
    }
}

static uint16_t Checksum (unsigned char *buf, int len)
{
    int i;
    uint16_t c;

    for (c=0, i=0; i < len; i++) {
        c += buf[i];
    }
    return (c);
}

static uint16_t CalCheckSum(uint32_t start, uint32_t len)
{
	int i;
	uint16_t lcksum = 0;
	
	for(i = 0; i < len; i+= FMC_FLASH_PAGE_SIZE)
	{
		ReadData(start + i, start + i + FMC_FLASH_PAGE_SIZE, (uint32_t*)aprom_buf);
		if(len - i >= FMC_FLASH_PAGE_SIZE)
			lcksum += Checksum(aprom_buf, FMC_FLASH_PAGE_SIZE);
		else
			lcksum += Checksum(aprom_buf, len - i);
	}
    
    return lcksum;
    
}

int ParseCmd(unsigned char *buffer, uint8_t len, BOOL bUSB)
{
	static uint32_t StartAddress, StartAddress_bak, TotalLen, TotalLen_bak, LastDataLen, g_packno = 1;
	uint8_t *response;
	uint16_t cksum, lcksum;
	uint32_t	lcmd, packno, srclen, i, regcnf0, security;
	unsigned char *pSrc;
	static uint32_t	gcmd;
	
	
	response = uart_sendbuf;

	pSrc = buffer;
	srclen = len;
	lcmd = inpw(pSrc);
	packno = inpw(pSrc + 4);
	outpw(response+4, 0);
	if((lcmd)&&(lcmd!=CMD_RESEND_PACKET))
		gcmd = lcmd;
	
	pSrc += 8;
	srclen -= 8;
	
	ReadData(Config0, Config0 + 8, (uint32_t*)(response+8));
	regcnf0 = *(uint32_t*)(response + 8);
	security = regcnf0 & 0x2;

	if(lcmd == CMD_SYNC_PACKNO)
	{
		g_packno = inpw(pSrc);
	}
	if((packno != g_packno) && ((lcmd != CMD_CONNECT))) //Connection state, packno always == 1
		goto out;

	if(lcmd == CMD_GET_FWVER)
		response[8] = FW_VERSION;//version 2.6
	else if(lcmd == CMD_GET_DEVICEID)
	{
		//outpw(response+8, g_pdid);
		outpw(response+8, SYS->PDID);
	}
	else if(lcmd == CMD_RUN_APROM || lcmd == CMD_RUN_LDROM || lcmd == CMD_RESET)
	{
		outpw(&SYS->RSTSTS, 3);//clear bit
		/* Set BS */
		if(lcmd == CMD_RUN_APROM)
		{
			i = (inpw(&FMC->ISPCTL) & 0xFFFFFFFC);
		}
		else if(lcmd == CMD_RUN_LDROM)
		{
			i = (inpw(&FMC->ISPCTL) & 0xFFFFFFFC);
			i |= 0x00000002;
		}
		else
		{
			i = (inpw(&FMC->ISPCTL) & 0xFFFFFFFE);//ISP disable
		}

		outpw(&FMC->ISPCTL, i);
		outpw(&SCB->AIRCR, (V6M_AIRCR_VECTKEY_DATA | V6M_AIRCR_SYSRESETREQ));

		/* Trap the CPU */
		while(1);
	}
	else if(lcmd == CMD_CONNECT)
	{
		g_connStatus = CONNECTED;
		g_packno = 1;
		g_enter_ISP = 1;
	}
	else if(lcmd == CMD_DISCONNECT)
	{
		g_connStatus = DISCONNECTED;
		g_enter_ISP = 0;
	}
		
	else if((lcmd == CMD_UPDATE_APROM) || (lcmd == CMD_ERASE_ALL))
	{
		if (lcmd == CMD_ERASE_ALL)                 //erase APROM + data flash
		{
			EraseAP(FALSE, 0, g_apromSize);        //erase all aprom including data flash
		}
		else
			EraseAP(TRUE, 0, 0);                   //don't erase data flash
		if(lcmd == CMD_ERASE_ALL)
		{
    		*(uint32_t*)(response + 8) = regcnf0|0x02;
    		UpdateConfig((uint32_t*)(response + 8), NULL);
		}
		else
			bUpdateApromCmd = TRUE;
	}
	else if(lcmd == CMD_GET_FLASHMODE)
	{
		//return 1: APROM, 2: LDROM
		outpw(response+8, (inpw(&FMC->ISPCTL)&0x2)? 2 : 1);
	}
	
	if((lcmd == CMD_UPDATE_APROM) || (lcmd == CMD_UPDATE_DATAFLASH))
	{
		if(lcmd == CMD_UPDATE_DATAFLASH)
		{
			StartAddress = g_dataFlashAddr;
		
			if(g_dataFlashSize)//g_dataFlashAddr
				EraseAP(FALSE, g_dataFlashAddr, g_dataFlashAddr + g_dataFlashSize);
			else goto out;
		}
		else
		{
			StartAddress = 0;
		}

		TotalLen = inpw(pSrc+4);
		pSrc += 8;
		srclen -= 8;
		StartAddress_bak = StartAddress;
		TotalLen_bak = TotalLen;

	}
	else if(lcmd == CMD_UPDATE_CONFIG)
	{
		if((security == 0) && (!bUpdateApromCmd))//security lock
			goto out;
		
		UpdateConfig((uint32_t*)(pSrc), (uint32_t*)(response+8));
		GetDataFlashInfo(&g_dataFlashAddr, &g_dataFlashSize);

		goto out;
	}
	else if (lcmd == CMD_RESEND_PACKET)
	{  	   
	   StartAddress -= LastDataLen;
	   TotalLen += LastDataLen;
		if((StartAddress & 0xFFE00) >= Config0)
			goto out;
		ReadData(StartAddress & 0xFFE00, StartAddress, (uint32_t*)aprom_buf);
		FMC_Erase(StartAddress & 0xFFE00);
		WriteData(StartAddress & 0xFFE00, StartAddress, (uint32_t*)aprom_buf);
		if((StartAddress%FMC_FLASH_PAGE_SIZE) >= (FMC_FLASH_PAGE_SIZE-LastDataLen))
	    	FMC_Erase((StartAddress & 0xFFE00)+FMC_FLASH_PAGE_SIZE);
		goto out;

	}
	
	if((gcmd == CMD_UPDATE_APROM) || (gcmd == CMD_UPDATE_DATAFLASH))
	{
		if(TotalLen >= srclen)
			TotalLen -= srclen;
		else{
			srclen = TotalLen;//prevent last package from over writing
			TotalLen = 0;
		}

		WriteData(StartAddress, StartAddress + srclen, (uint32_t*)pSrc);
		//memset(pSrc, 0, srclen);
//		{
//			uint32_t *p = (uint32_t*)pSrc;
//			for(i = 0; i<srclen/4; i++)
//				p[i] = 0;
//		}
		
		ReadData(StartAddress, StartAddress + srclen, (uint32_t*)pSrc);
		StartAddress += srclen;
		LastDataLen =  srclen;
		
		if(TotalLen == 0)
		{
			lcksum = CalCheckSum(StartAddress_bak, TotalLen_bak);
			outps(response + 8, lcksum);
		}
	}
out:
	cksum = Checksum(buffer, len);
	outps(response, cksum);
	++g_packno;
	outpw(response+4, g_packno);
	g_packno++;

	return 0;
}

//the smallest of APROM size is 2K 
static uint32_t GetApromSize()
{
	uint32_t size = 0x800, data;
	int result;
	
	do{
		result = FMC_Read(size, &data);
		if(result < 0)
	    {
    		return size;
		}
		else 
			size *= 2;
	}while(1);
}

/*---------------------------------------------------------------------------------------------------------*/
/* ISR to handle UART Channel 0 interrupt event                                                            */
/*---------------------------------------------------------------------------------------------------------*/
void UART0_IRQHandler(void)
{
    /*----- Determine interrupt source -----*/
	
		if (inpw(&UART0->INTSTS) & 0x11) //RDA FIFO interrupt or RDA timeout interrupt 
   		{
   			while(((inpw(&UART0->FIFOSTS) & (0x4000)) == 0) && (bufhead < 64))//RX fifo not empty
				uart_rcvbuf[bufhead++] = inpw(&UART0->DAT);
		}
	
		WDT->CTL |= WDT_CTL_RSTCNT_Msk;//WDT_RESET_COUNTER();
     WDT->CTL |= WDT_CTL_WDTEN_Msk;//WDT_ENABLE_COUNTING;
	    
	
	  if(bufhead == 64)
	  {
        if(g_connStatus == DISCONNECTED)
        {
            // check first command is connect
            if(uart_rcvbuf[0] == CMD_CONNECT)
            {
                bUartDataReady = TRUE;
                bufhead = 0;
            }
            else
            {
                bUartDataReady = FALSE;
                bufhead = 0;
            }
        }
        else
        {
            bUartDataReady = TRUE;
            bufhead = 0;
        }
    }
}

void UART_Init()
{
/*---------------------------------------------------------------------------------------------------------*/
/* Init UART                                                                                               */
/*---------------------------------------------------------------------------------------------------------*/     

   // 22.1184 MHz reference clock input, for 115200 bps
   // 22118400 / 115200 = 192. Using mode 2 to calculate baudrate, 192 - 2 = 190 = 0xBE
   UART0->BAUD = UART_BAUD_BAUDM1_Msk | UART_BAUD_BAUDM0_Msk | (0xBE);	 
    
// 	 SYS->IPRST1 = SYS_IPRST1_UART0RST_Msk;
// 	 SYS->IPRST1 &= ~SYS_IPRST1_UART0RST_Msk;
	
		// Set UART to 8 bit character length, 1 stop bit, and no parity 
    UART0->LINE = UART_LINE_WLS_Msk;	
	  outpw(&UART0->FIFO, 0x32); //14
		outpw(&UART0->TOUT, 0x40);
		
    outpw(&UART0->INTEN, 0x811);
    NVIC_EnableIRQ(UART0_IRQn);
}

/*---------------------------------------------------------------------------------------------------------*/
/* MAIN function                                                                                           */
/*---------------------------------------------------------------------------------------------------------*/
int main(void)
{
	
    /* Unlock protected registers */
    SYS->REGLCTL = 0x59;
    SYS->REGLCTL = 0x16;
    SYS->REGLCTL = 0x88;
    
    /*  Read User Config to select internal high speed RC  HIRC 22M */
    // Factory Default is internal high speed RC 44.2368M divided by 2 
		SystemInit();
	
    /* Enable HIRC */
    CLK->PWRCTL = CLK_PWRCTL_HIRCEN_Msk;

    /* Waiting for clock ready */
    while(!(CLK->STATUS & CLK_STATUS_HIRCSTB_Msk));
	
    /* Enable UART clock */
    CLK->APBCLK = CLK_APBCLK_UART0CKEN_Msk;
	
    /* Set P1 multi-function pins for UART RXD, TXD */
    SYS->P0_MFP = SYS_MFP_P00_TXD | SYS_MFP_P01_RXD;
    

    FMC->ISPCTL |=  FMC_ISPCTL_ISPEN_Msk;

    
    // ==================== WDT ================ //
		// Data drop for size less than 64 byte
	  /* Enable LIRC 10K (fro WDT) */
    CLK->PWRCTL = CLK_PWRCTL_LIRCEN_Msk;
    /* Waiting for clock ready */
    while(!(CLK->STATUS & CLK_STATUS_LIRCSTB_Msk));
	 
    CLK->APBCLK |= CLK_APBCLK_WDTCKEN_Msk;
	  							
    WDT->CTL = (1UL << 8) | WDT_CTL_INTEN_Msk | WDT_CTL_WDTEN_Msk |
		   				 WDT_CTL_RSTCNT_Msk | WDT_CTL_IF_Msk;
							
    NVIC_EnableIRQ(WDT_IRQn);
    
	  
		g_apromSize = GetApromSize();
	  GetDataFlashInfo(&g_dataFlashAddr, &g_dataFlashSize);
    
    UART_Init();
    g_connStatus = DISCONNECTED;
    g_enter_ISP = 0;
		
    UART_Init();       	

    //timeout 50ms    
	  SysTick->LOAD = 50000 * 22; /* using 22Hz cpu clock*/
    SysTick->VAL   =  (0x00);
    SysTick->CTRL = SysTick->CTRL|(1 << SysTick_CTRL_CLKSOURCE_Pos) 
					| (1<<SysTick_CTRL_ENABLE_Pos);//using cpu clock
	
					
    while(1)
    {
				if(bUartDataReady == TRUE)
				{
						g_timecnt = 0;
						ParseCmd(uart_rcvbuf, PACKET_SIZE, FALSE);
						PutString();
						bUartDataReady = FALSE;
				}

						
				if((!g_enter_ISP) && (SysTick->CTRL & (1 << 16)) != 0)     
						goto _APROM;            
    }
    
    
_APROM:    
    outpw(&SYS->RSTSTS, 3);//clear bit
    outpw(&FMC->ISPCTL, inpw(&FMC->ISPCTL) & 0xFFFFFFFC);
    // 	   		outpw(&SCB->AIRCR, (V6M_AIRCR_VECTKEY_DATA | V6M_AIRCR_SYSRESETREQ));//SYSRESETREQ
    outpw(&SCB->AIRCR, (V6M_AIRCR_VECTKEY_DATA | V6M_AIRCR_SYSRESETREQ));//SYSRESETREQ
    //NVIC_SystemReset();

    /* Trap the CPU */
    while(1);		
}
