/******************************************************************************
 * @file     CRC.h
 * @version  V1.0
 * $Revision: 1 $
 * $Date: 14/09/09 4:43p $
 * @brief    NUC123 Series CRC Driver Header File
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#ifndef __CRC_H__
#define __CRC_H__

#include "NUC123Series.h"

/*---------------------------------------------------------------------------------------------------------*/
/*  CRC_CTL constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define CRC_CTL_CRC_CCITT               (0UL<<CRC_CTL_CRC_MODE_Pos) /*!< CRC polynomial mode is CRC_CCITT */
#define CRC_CTL_CRC_8                   (1UL<<CRC_CTL_CRC_MODE_Pos) /*!< CRC polynomial mode is CRC_8 */
#define CRC_CTL_CRC_16                  (2UL<<CRC_CTL_CRC_MODE_Pos) /*!< CRC polynomial mode is CRC_16 */
#define CRC_CTL_CRC_32                  (3UL<<CRC_CTL_CRC_MODE_Pos) /*!< CRC polynomial mode is CRC_32 */

#define CRC_CTL_CPU_WDLEN_8             (0UL<<CRC_CTL_CPU_WDLEN_Pos) /*!< When operation in CPU PIO mode(CRCCEN= 1, TRIG_EN = 0), write data length is 8-bit */
#define CRC_CTL_CPU_WDLEN_16            (1UL<<CRC_CTL_CPU_WDLEN_Pos) /*!< When operation in CPU PIO mode(CRCCEN= 1, TRIG_EN = 0), write data length is 16-bit */
#define CRC_CTL_CPU_WDLEN_32            (2UL<<CRC_CTL_CPU_WDLEN_Pos) /*!< When operation in CPU PIO mode(CRCCEN= 1, TRIG_EN = 0), write data length is 32-bit */

#define CRC_CTL_CHECKSUM_COMPLEMENT     (1UL<<CRC_CTL_CHECKSUM_COM_Pos) /*!< Ones complement for CRC checksum */
#define CRC_CTL_CHECKSUM_NORMAL         (0UL<<CRC_CTL_CHECKSUM_COM_Pos) /*!< No Ones complement for CRC checksum */

#define CRC_CTL_WDATA_COMPLEMENT        (1UL<<CRC_CTL_WDATA_COM_Pos) /*!< Ones complement for CRC write data */
#define CRC_CTL_WDATA_NORMAL            (0UL<<CRC_CTL_WDATA_COM_Pos) /*!< No Ones complement for CRC write data */

#define CRC_CTL_CHECKSUM_ORDER_REVERSE  (1UL<<CRC_CTL_CHECKSUM_RVS_Pos) /*!< Bit order reverse for CRC checksum(the whole word) */
#define CRC_CTL_CHECKSUM_ORDER_NORMAL   (0UL<<CRC_CTL_CHECKSUM_RVS_Pos) /*!< No bit order reverse for CRC checksum */

#define CRC_CTL_WDATA_ORDER_BYTE_RVS    (1UL<<CRC_CTL_WDATA_RVS_Pos) /*!< Bit order reverse for CRC write data(per byte) */
#define CRC_CTL_WDATA_ORDER_NORMAL      (0UL<<CRC_CTL_WDATA_RVS_Pos) /*!< No bit order reverse for write data */

#define CRC_CTL_TRIG_EN                 (1UL<<CRC_CTL_TRIG_EN_Pos) /*!< Enable CRC DMA data read or write transfer */
#define CRC_CTL_TRIG_DIS                (0UL<<CRC_CTL_TRIG_EN_Pos) /*!< Disable CRC DMA data read or write transfer */

#define CRC_CTL_CRC_RST                 (1UL<<CRC_CTL_CRC_RST_Pos) /*!< Reset internal CRC state machine and buffer. Contents of control register will not be cleared */

#define CRC_CTL_CRC_CH_EN               (1UL<<CRC_CTL_CRCCEN_Pos) /*!< Enables CRCs operation */
#define CRC_CTL_CRC_CH_DIS              (0UL<<CRC_CTL_CRCCEN_Pos) /*!< Enables CRCs operation */

/*---------------------------------------------------------------------------------------------------------*/
/*  CRC_DMAIER constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define CRC_DMAIER_BLKD_INT_EN          (1UL<<CRC_DMAIER_CRC_BLKD_IE_Pos) /*!< Enable CRC PDMA transfer done interrupt */
#define CRC_DMAIER_BLKD_INT_DIS         (0UL<<CRC_DMAIER_CRC_BLKD_IE_Pos) /*!< Disable CRC PDMA transfer done interrupt */

#define CRC_DMAIER_TABORT_INT_EN        (1UL<<CRC_DMAIER_CRC_TABORT_IE_Pos) /*!< Enable CRC PDMA transfer target abort interrupt */
#define CRC_DMAIER_TABORT_INT_DIS       (0UL<<CRC_DMAIER_CRC_TABORT_IE_Pos) /*!< Disable CRC PDMA transfer target abort interrupt */

/*---------------------------------------------------------------------------------------------------------*/
/*  CRC_DMAISR constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define CRC_DMAISR_BLKD_IF                (1UL<<PDMA_ISR_BLKD_IF_Pos) /*!< CRC PDMA transfer done interrupt flag */
#define CRC_DMAISR_TABORT_IF              (1UL<<PDMA_ISR_TABORT_IF_Pos) /*!< CRC PDMA transfer target abort interrupt flag */

/*---------------------------------------------------------------------------------------------------------*/
/* Macro functions                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
/**
 * @brief       Enable CRC controller clock
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Enable CRC controller clock
 */
#define CRC_CLOCK_ENABLE()                     (PDMA_GCR->GCRCSR |= PDMA_GCRCSR_CRC_CLK_EN)


/**
 * @brief       Disable CRC controller clock
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Disable CRC controller clock
 */
#define CRC_CLOCK_DISABLE()                    (PDMA_GCR->GCRCSR = PDMA_GCR->GCRCSR & (~PDMA_GCRCSR_CRC_CLK_EN_Msk) | PDMA_GCRCSR_CRC_CLK_DIS)


/**
 * @brief       Get CRC controller interrupt status
 *
 * @param       None
 *
 * @retval      0   No interrupt at CRC controller
 * @retval      1   CRC controller generated an interrupt
 *
 * @details     Get CRC controller interrupt status
 */
#define CRC_GET_INTR_STATUS()                  ((PDMA_GCR->GCRISR & PDMA_GCRISR_INTRCRC_STATUS)?1:0)


/**
 * @brief       Specify CRC polynomial mode
 *
 * @param[in]   mode  Selectable CRC polynomial mode. It could be \n
 *                    CRC_CTL_CRC_CCITT, CRC_CTL_CRC_8, CRC_CTL_CRC_16, CRC_CTL_CRC_32
 *
 * @return      None
 *
 * @details     Specify CRC polynomial mode for CRC operation
 */
#define CRC_SET_POLYNO_MODE(mode)              (CRC->CTL = CRC->CTL & (~CRC_CTL_CRC_MODE_Msk) | (mode))


/**
 * @brief       Set CRC CPU write data length
 *
 * @param[in]   len  Selectable CPU write data length. It could be \n
 *                   CRC_CTL_CPU_WDLEN_8, CRC_CTL_CPU_WDLEN_16, CRC_CTL_CPU_WDLEN_32
 *
 * @return      None
 *
 * @details     When CRC operating in CPU PIO mode, set CPU write data length
 */
#define CRC_SET_CPU_WRITE_DATA_LEN(len)        (CRC->CTL = CRC->CTL & (~CRC_CTL_CPU_WDLEN_Msk) | (len))


/**
 * @brief       Set CRC checksum one's complement
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Set one's complement for CRC checksum
 */
#define CRC_SET_CHECKSUM_COMPLEMENT()          (CRC->CTL = CRC->CTL & (~CRC_CTL_CHECKSUM_COM_Msk) | CRC_CTL_CHECKSUM_COMPLEMENT)


/**
 * @brief       CRC checksum non one's complement
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Non one's complement for CRC checksum
 */
#define CRC_SET_CHECKSUM_NON_COMPLEMENT()      (CRC->CTL = CRC->CTL & (~CRC_CTL_CHECKSUM_COM_Msk) | CRC_CTL_CHECKSUM_NORMAL)


/**
 * @brief       Set CRC checksum bit order reverse
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Set the whole bit field of CRC checksum bit order reverse
 */
#define CRC_SET_CHECKSUM_BIT_ORDER_RVS()       (CRC->CTL = CRC->CTL & (~CRC_CTL_CHECKSUM_RVS_Msk) | CRC_CTL_CHECKSUM_ORDER_REVERSE)


/**
 * @brief       Set CRC checksum bit order non-reverse
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Set the whole bit field of CRC checksum in normal bit order
 */
#define CRC_SET_CHECKSUM_BIT_ORDER_NON_RVS()   (CRC->CTL = CRC->CTL & (~CRC_CTL_CHECKSUM_RVS_Msk) | CRC_CTL_CHECKSUM_ORDER_NORMAL)


/**
 * @brief       Set CRC write data one's complement
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Set one's complement for CRC write data
 */
#define CRC_SET_WDATA_COMPLEMENT()             (CRC->CTL = CRC->CTL & (~CRC_CTL_WDATA_COM_Msk) | CRC_CTL_WDATA_COMPLEMENT)


/**
 * @brief       CRC write non one's complement
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Non one's complement for CRC write data
 */
#define CRC_SET_WDATA_NON_COMPLEMENT()         (CRC->CTL = CRC->CTL & (~CRC_CTL_WDATA_COM_Msk) | CRC_CTL_WDATA_NORMAL)


/**
 * @brief       Set CRC write data bit order reverse
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Set CRC write data bit order reverse in per byte
 */
#define CRC_SET_WDATA_BIT_ORDER_BYTE_RVS()     (CRC->CTL = CRC->CTL & (~CRC_CTL_WDATA_RVS_Msk) | CRC_CTL_WDATA_ORDER_BYTE_RVS)


/**
 * @brief       Set CRC write data bit order non-reverse
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Set CRC write data in normal bit order
 */
#define CRC_SET_WDATA_BIT_ORDER_NON_RVS()      (CRC->CTL = CRC->CTL & (~CRC_CTL_WDATA_RVS_Msk) | CRC_CTL_WDATA_ORDER_NORMAL)


/**
 * @brief       CRC PDMA triger enable
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Enable CRC DMA data read or write transfer
 */
#define CRC_PDMA_TRIG_ENABLE()                 (CRC->CTL = CRC->CTL & (~CRC_CTL_TRIG_EN_Msk) | CRC_CTL_TRIG_EN)


/**
 * @brief       Get CRC PDMA transfer status
 *
 * @param       None
 *
 * @retval      0   CRC PDMA transfer is completed
 * @retval      1   CRC PDMA transfer is not yet completed
 *
 * @details     Get CRC PDMA transfer status.\n
 *              When CRC DMA transfer complete, this bit will be cleared automatically
 */
#define CRC_PDMA_GET_TRANS_STATUS()            ((CRC->CTL & CRC_CTL_TRIG_EN_Msk)?1:0)


/**
 * @brief       CRC engine reset
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Reset the internal CRC state machine and internal buffer. \n
 *              The contents of control register will not be cleared
 */
#define CRC_ENGINE_RESET()                     (CRC->CTL = CRC->CTL & (~CRC_CTL_CRC_RST_Msk) | CRC_CTL_CRC_RST)


/**
 * @brief       CRC channel enable
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Enable CRC's operation
 */
#define CRC_CRC_CH_ENABLE()                    (CRC->CTL = CRC->CTL & (~CRC_CTL_CRCCEN_Msk) | CRC_CTL_CRC_CH_EN)


/**
 * @brief       CRC channel disable
 *
 * @param       None
 *
 * @return      None
 *
 * @details     When operating in CRC DMA mode(TRIG_EN = 1), if user clears this bit, \n
 *              the DMA operation will be continuous until all CRC DMA operation is done,\n
 *              and the TRIG_EN bit will asserted until all CRC DMA operation done. \n
 *              But in this case, the CRC_DMAISR[BLKD_IF] flag will inactive, \n
 *              user can read CRC result by reading CRC_CHECKSUM register when TRIG_EN = 0. \n
 *              When operating in CRC DMA mode(TRIG_EN = 1), if user wants to stop the transfer immediately, \n
 *              user can write 1 to CRC_RST bit to stop the transmission
 */
#define CRC_CRC_CH_DISABLE()                   (CRC->CTL = CRC->CTL & (~CRC_CTL_CRCCEN_Msk) | CRC_CTL_CRC_CH_DIS)


/**
 * @brief       Set CRC PDMA transfer source address
 *
 * @param[in]   addr  CRC PDMA transfer 32-bit address, must word aligned
 *
 * @return      None
 *
 * @details     Set CRC PDMA transfer source address
 */
#define CRC_PDMA_SET_SOURCE_ADDR(addr)         (CRC->DMASAR = ((addr) & 0xFFFFFFFCUL))


/**
 * @brief       Set CRC PDMA transfer byte count
 *
 * @param[in]   cnt  CRC PDMA transfer 16-bit byte count
 *
 * @return      None
 *
 * @details     Set CRC PDMA transfer byte count
 */
#define CRC_PDMA_SET_TRANS_COUNT(cnt)          (CRC->DMABCR = (cnt))


/**
 * @brief       Get CRC PDMA current source address
 *
 * @param       None
 *
 * @return      Source address where the CRC DMA transfer just occurs
 *
 * @details     Get CRC PDMA current source address
 */
#define CRC_PDMA_GET_CURRENT_SOURCE_ADDR()     (CRC->DMACSAR)


/**
 * @brief       Get CRC PDMA current byte count
 *
 * @param       None
 *
 * @return      The current remained byte count of CRC_DMA
 *
 * @details     Get CRC PDMA current byte count
 */
#define CRC_PDMA_GET_CURRENT_TRANS_COUNT()     (CRC->DMACBCR)


/**
 * @brief       Enable CRC PDMA transfer done interrupt
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Enable CRC PDMA transfer done interrupt
 */
#define CRC_PDMA_TRANS_DONE_INT_ENABLE()       (CRC->DMAIER |= CRC_DMAIER_BLKD_INT_EN)


/**
 * @brief       Disable CRC PDMA transfer done interrupt
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Disable CRC PDMA transfer done interrupt
 */
#define CRC_PDMA_TRANS_DONE_INT_DISABLE()      (CRC->DMAIER = CRC->DMAIER & (~CRC_DMAIER_CRC_BLKD_IE_Msk) | CRC_DMAIER_BLKD_INT_DIS)


/**
 * @brief       Get CRC PDMA transfer done interrupt status
 *
 * @param       None
 *
 * @retval      0   CRC DMA has not finished all transfer
 * @retval      1   CRC DMA has finished all transfer
 *
 * @details     Get CRC PDMA transfer done interrupt status
 */
#define CRC_PDMA_GET_TRANS_DONE_INT_STATUS()   ((CRC->DMAISR & CRC_DMAISR_BLKD_IF)?1:0)


/**
 * @brief       Clear CRC PDMA transfer done interrupt status
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Clear CRC PDMA transfer done interrupt status
 */
#define CRC_PDMA_CLR_TRANS_DONE_INT_STATUS()   (CRC->DMAISR = CRC_DMAISR_BLKD_IF)


/**
 * @brief       Enable CRC PDMA target abort interrupt
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Enable CRC PDMA target abort interrupt
 */
#define CRC_PDMA_TARGET_ABORT_INT_ENABLE()     (CRC->DMAIER |= CRC_DMAIER_TABORT_INT_EN)


/**
 * @brief       Disable CRC PDMA target abort interrupt
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Disable CRC PDMA target abort interrupt
 */
#define CRC_PDMA_TARGET_ABORT_INT_DISABLE()    (CRC->DMAIER = CRC->DMAIER & (~CRC_DMAIER_CRC_TABORT_IE_Msk) | CRC_DMAIER_TABORT_INT_DIS)


/**
 * @brief       Get CRC PDMA target abort interrupt status
 *
 * @param       None
 *
 * @retval      0   No bus ERROR response received
 * @retval      1   Bus ERROR response received
 *
 * @details     Get CRC PDMA target abort interrupt status
 */
#define CRC_PDMA_GET_TARGET_ABORT_INT_STATUS() ((CRC->DMAISR & CRC_DMAISR_TABORT_IF)?1:0)


/**
 * @brief       Clear CRC PDMA target abort interrupt status
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Clear RCR PDMA target abort interrupt status
 */
#define CRC_PDMA_CLR_TARGET_ABORT_INT_STATUS() (CRC->DMAISR = CRC_DMAISR_TABORT_IF)


/**
 * @brief       Write CRC data register to perform CRC operation
 *
 * @param[in]   data  32-bit data to be written to CRC data register
 *
 * @return      None
 *
 * @details     When operated in CPU PIO mode, write CRC data register to perform CRC operation
 */
#define CRC_WRITE_DATA(data)                   (CRC->WDATA = (data))


/**
 * @brief       Get CRC PDMA internal buffer
 *
 * @param       None
 *
 * @return      DMA internal buffer
 *
 * @details     When operated in CRC DMA mode, CRC_WDATA register used as DMA internal buffer
 */
#define CRC_PDMA_GET_INTERNAL_BUF()            (CRC->WDATA)


/**
 * @brief       Set CRC seed
 *
 * @param[in]   data  32-bit data to be used as CRC seed
 *
 * @return      None
 *
 * @details     Set CRC seed
 */
#define CRC_SET_CRC_SEED(data)                 (CRC->SEED = (data))


/**
 * @brief       Get CRC seed
 *
 * @param       None
 *
 * @return      CRC seed value
 *
 * @details     Get CRC seed
 */
#define CRC_GET_CRC_SEED()                     (CRC->SEED)


/**
 * @brief       Get CRC checksum
 *
 * @param       None
 *
 * @return      CRC checksum value
 *
 * @details     Get CRC checksum
 */
#define CRC_GET_CRC_CHECKSUM()                 (CRC->CHECKSUM)

#endif
