/**************************************************************************//**
 * @file     I2C.h   
 * @version  V2.1
 * $Revision: 1 $
 * $Date: 14/09/09 4:43p $
 * @brief    NUC123 Series I2C Driver Header File
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#ifndef __I2C_H__
#define __I2C_H__

#include "NUC123Series.h"


/**
 * @brief      Set I2C Control Bits
 *
 * @param[in]  port    I2C port
 *                     -I2C0
 *                     -I2C1
 * @param[in]  u8Ctrl  I2C Control Bit. Including
 *                     -I2C_I2CON_STA_STO_SI   : Set STA, STO and SI bits
 *                     -I2C_I2CON_STA_STO_SI_AA: Set STA, STO, SI and AA bits 
 *                     -I2C_I2CON_STA_SI       : Set STA and SI bits      
 *                     -I2C_I2CON_STA_SI_AA    : Set STA, SI and AA bits
 *                     -I2C_I2CON_STO_SI       : Set STO and SI bits        
 *                     -I2C_I2CON_STO_SI_AA    : Set STO, SI and AA bits    
 *                     -I2C_I2CON_SI           : Set SI bit               
 *                     -I2C_I2CON_SI_AA        : Set SI and AA bits
 *                     -I2C_I2CON_STA          : Set STA bit               
 *                     -I2C_I2CON_AA           : Set AA bit
 *
 * @return     None
 *
 * @details    To set I2C control bits include STA, STO, SI and AA in I2C CONTROL REGISTER (I2CON)
 *			   Example: I2C_SET_CONTROL_BITS(I2C0,I2C_I2CON_STA_STO_SI)
 * @note
 *             This function will enable EI bit. If user does not need to use interrupt, 
 *             user can disable corresponding NVIC bit of I2C.
 */
#define I2C_SET_CONTROL_BITS(port, u8Ctrl)     ((port)->I2CON = I2C_I2CON_EI_Msk | I2C_I2CON_ENS1_Msk | (u8Ctrl))

/**
 * @brief      Generate START signal to I2C bus
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    This macro is used to generate START signal to specified I2C port 
 *			   Example: I2C_START(I2C0)
 */
#define I2C_START(port) I2C_SET_CONTROL_BITS(port, I2C_I2CON_STA_SI)

/**
 * @brief      Clear START
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    Clear START. 
 *             Example: I2C_CLEAR_START(I2C0)
 */
#define I2C_CLEAR_START(port) (port)->I2CON &= ~I2C_I2CON_STA

/**
 * @brief      Generate STOP signal to I2C bus
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    This macro is used to generate STOP signal to specified I2C port  
 *             Example: I2C_STOP(I2C0)
 */
#define I2C_STOP(port) I2C_SET_CONTROL_BITS(port, I2C_I2CON_STO_SI)

/**
 * @brief      Clear STOP
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    Clear START. 
 *             Example: I2C_CLEAR_STOP(I2C0)
 */
#define I2C_CLEAR_STOP(port) (port)->I2CON &= ~I2C_I2CON_STO

/**
 * @brief      Enable I2C interrupt
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    Enable I2C interrupt. 
 *             Example: I2C_ENABLE_I2C_INT(I2C0)
 */
#define I2C_ENABLE_I2C_INT(port) (port)->I2CON |= I2C_I2CON_EI

/**
 * @brief      Disable I2C interrupt
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    Disable I2C interrupt. 
 *             Example: I2C_DISABLE_I2C_INT(I2C0)
 */
#define I2C_DISABLE_I2C_INT(port) (port)->I2CON &= ~I2C_I2CON_EI

/**
 * @brief      Enable I2C
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    Enable I2C. 
 *             Example: I2C_ENABLE_I2C(I2C0)
 */
#define I2C_ENABLE_I2C(port) (port)->I2CON |= I2C_I2CON_ENSI

/**
 * @brief      Disable I2C
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    Disable I2C. 
 *             Example: I2C_DISABLE_I2C(I2C0)
 */
#define I2C_DISABLE_I2C(port) (port)->I2CON &= ~I2C_I2CON_ENSI


/**
 * @brief      Waiting for I2C action done
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    This function is used to wait I2C action done by polling SI bit of I2CON. 
 *             Example: I2C_WAIT_READY(I2C0)
 */
#define I2C_WAIT_READY(port) while(((port)->I2CON & I2C_I2CON_SI) == 0)

/**
 * @brief      Clear SI
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    Clear SI to start action. 
 *             Example: I2C_CLEAR_SI(I2C0)
 */
#define I2C_CLEAR_SI(port) (port)->I2CON |= I2C_I2CON_SI

/**
 * @brief      Set AA
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    An acknowledged (low level to SDA) will be returned during the acknowledge clock pulse on the SCL line when 1 
 *             Example: I2C_SET_AA(I2C0)
 */
#define I2C_SET_AA(port) (port)->I2CON |= I2C_I2CON_AA

/**
 * @brief      Clear AA
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    No acknowledged (low level to SDA) will be returned
 *             Example: I2C_CLEAR_AA(I2C0)
 */
#define I2C_CLEAR_AA(port) (port)->I2CON &= ~I2C_I2CON_AA

/**
 * @brief      Set I2C data
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    Set I2C data
 *             Example: I2C_SET_DATA(I2C0,0x55)
 */
#define I2C_SET_DATA(port,u8data) (port)->I2CDAT = u8data

/**
 * @brief      Get I2C data	status
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    Get I2C data	status
 *             Example: I2C_GET_I2CSTATUS(I2C0)
 */
#define I2C_GET_I2CSTATUS(port) ((port)->I2CSTATUS)

/**
 * @brief      Set I2C divider
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    Set I2C divider
 *             Example: I2C_SET_DIVIDER(I2C0,0x05)
 */
#define I2C_SET_DIVIDER(port,u8div) (port)->I2CDAT |= u8div

/**
 * @brief      Enable I2C time out counter
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    Enable I2C time out counter 
 *             Example: I2C_ENABLE_TOC(I2C0)
 */
#define I2C_ENABLE_TOC(port) (port)->I2CTOC |= I2C_I2CTOC_ENTI

/**
 * @brief      Disable I2C time out counter
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    Disable I2C time out counter 
 *             Example: I2C_DISABLE_TOC(I2C0)
 */
#define I2C_DISABLE_TOC(port) (port)->I2CTOC &= ~I2C_I2CTOC_ENTI

/**
 * @brief      Enable I2C time out counter clock Divided by 4
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    When Enabled, the time-out period is extend 4 times.
 *             Example: I2C_ENABLE_TOC_DIV4(I2C0)
 */
#define I2C_ENABLE_TOC_DIV4(port) (port)->I2CTOC |= I2C_I2CTOC_DIV4

/**
 * @brief      Disable I2C time out counter clock Divided by 4
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    Disable I2C time out counter clock Divided by 4
 *             Example: I2C_DISABLE_TOC_DIV4(I2C0)
 */
#define I2C_DISABLE_TOC_DIV4(port) (port)->I2CTOC &= ~I2C_I2CTOC_DIV4

/**
 * @brief      Read time out flag
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @retval     0 =  No time out happened
 * @retval     1 =  Time out happened. 
 *
 * @details    Read time out flag. Time-out flag is set by H/W. It can interrupt CPU
 *             Example: I2C_READ_TIMEOUT_FLAG(I2C0)
 */
#define I2C_READ_TIMEOUT_FLAG(port) (((port)->I2CTOC &I2C_I2CTOC_TIF)>>I2C_I2CTOC_TIF_Pos)

/**
 * @brief      Clear time out flag
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    Clear time out flag
 *             Example: I2C_CLEAR_TIMEOUT_FLAG(I2C0)
 */
#define I2C_CLEAR_TIMEOUT_FLAG(port) (port)->I2CTOC &= ~I2C_I2CTOC_TIF

/**
 * @brief      Enable General Call Mode
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    Enable General Call Mode.
 *             Example: I2C_ENABLE_GENERAL_CALL(I2C0)
 */
#define I2C_ENABLE_GENERAL_CALL(port)    ((port)->I2CADDR0 |=  I2C_I2CADDR_GC) /*!< Enable General Call Mode  */

/**
 * @brief      Disable General Call Mode
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    Disable General Call Mode.
 *             Example: I2C_DISABLE_GENERAL_CALL(I2C0)
 */
#define I2C_DISABLE_GENERAL_CALL(port)   ((port)->I2CADDR0 &= ~I2C_I2CADDR_GC) /*!< Disable General Call Mode */


/**
 * @brief      Set I2C address
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @param[in]  u8add  7-bit slave address
 *
 * @return     None
 *
 * @details    Set I2C address 0
 *             Example: I2C_SET_ADDRESS0(I2C0,0xA0)
 */
#define I2C_SET_ADDRESS0(port,u8add) (port)->I2CADDR0 = u8add

/**
 * @brief      Set I2C address
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @param[in]  u8add  7-bit slave address
 * @return     None
 *
 * @details    Set I2C address 1
 *             Example: I2C_SET_ADDRESS1(I2C0,0xB0)
 */
#define I2C_SET_ADDRESS1(port,u8add) (port)->I2CADDR1 = u8add

/**
 * @brief      Set I2C address
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @param[in]  u8add  7-bit slave address
 *
 * @return     None
 *
 * @details    Set I2C address 2
 *             Example: I2C_SET_ADDRESS2(I2C0,0xC0)
 */
#define I2C_SET_ADDRESS2(port,u8add) (port)->I2CADDR2 = u8add

/**
 * @brief      Set I2C address
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @param[in]  u8add  7-bit slave address
 *
 * @return     None
 *
 * @details    Set I2C address 3
 *             Example: I2C_SET_ADDRESS3(I2C0,0xD0)
 */
#define I2C_SET_ADDRESS3(port,u8add) (port)->I2CADDR3 = u8add


/**
 * @brief      Set I2C address mask
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 *
 * @param[in]  u8addm  7-bit data to mask the corresponding slave address bit(s) of I2CADDR0
 *
 * @return     None
 *
 * @details    Set I2C address mask0
 *             Example: I2C_SET_ADDRESS_MASK0(I2C0,0x20)
 */
#define I2C_SET_ADDRESS_MASK0(port,u8addm) (port)->I2CADM0 = u8addm

/**
 * @brief      Set I2C address mask
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @param[in]  u8addm  7-bit data to mask the corresponding slave address bit(s) of I2CADDR1
 *
 * @return     None
 *
 * @details    Set I2C address mask1
 *             Example: I2C_SET_ADDRESS_MASK1(I2C0,0x40)
 */
#define I2C_SET_ADDRESS_MASK1(port,u8addm) (port)->I2CADM1 = u8addm

/**
 * @brief      Set I2C address mask
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @param[in]  u8addm  7-bit data to mask the corresponding slave address bit(s) of I2CADDR2
 *
 * @return     None
 *
 * @details    Set I2C address mask2
 *             Example: I2C_SET_ADDRESS_MASK2(I2C0,0x80)
 */
#define I2C_SET_ADDRESS_MASK2(port,u8addm) (port)->I2CADM2 = u8addm

/**
 * @brief      Set I2C address mask
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @param[in]  u8addm  7-bit data to mask the corresponding slave address bit(s) of I2CADDR3
 *
 * @return     None
 *
 * @details    Set I2C address mask3
 *             Example: I2C_SET_ADDRESS_MASK3(I2C0,0x10)
 */
#define I2C_SET_ADDRESS_MASK3(port,u8addm) (port)->I2CADM3 = u8addm

/**
 * @brief      I2C Wake-up Function Enable
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    I2C Wake-up Function Enable.
 *             Example: I2C_ENABLE_WAKEUP(I2C0)
 */
#define I2C_ENABLE_WAKEUP(port) (port)->I2WKUPCON |= I2C_I2CWKUPCON_WKUPEN

/**
 * @brief      I2C Wake-up Function Disable
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None
 *
 * @details    I2C Wake-up Function Disable.
 *             Example: I2C_DISABLE_WAKEUP(I2C0)
 */
#define I2C_DISABLE_WAKEUP(port) (port)->I2WKUPCON &= ~I2C_I2CWKUPCON_WKUPEN

/**
 * @brief      Read wake up Interrupt flag
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @retval     0 = Wake-up flag inactive. 
 * @retval     1 = Wake-up flag active. 
 *
 * @details    Read wake up Interrupt flag
 *             Example: I2C_READ_WAKEUP_FLAG(I2C0)
 */
#define I2C_READ_WAKEUP_FLAG(port) ((port)->I2CWKUPSTS & I2C_I2CWKUPCON_WKUPIF)

/**
 * @brief      Clear wake up Interrupt flag
 *
 * @param[in]  port    I2C port 
 *                     -I2C0
 *                     -I2C1
 * @return     None 
 *
 * @details    Clear wake up Interrupt flag
 *             Example: I2C_CLEAR_WAKEUP_FLAG(I2C0)
 */
#define I2C_CLEAR_WAKEUP_FLAG(port) (port)->I2CWKUPSTS &= ~I2C_I2CWKUPCON_WKUPIF

/*---------------------------------------------------------------------------------------------------------*/
/*  I2CON constant definitions.                                                                            */
/*---------------------------------------------------------------------------------------------------------*/
#define I2C_I2CON_EI          (1ul << I2C_I2CON_EI_Pos)	   /*!< Enable Interrupt */
#define I2C_I2CON_ENSI        (1ul << I2C_I2CON_ENS1_Pos)  /*!< I2C Controller Enable Bit  */
#define I2C_I2CON_STA         (1ul << I2C_I2CON_STA_Pos)   /*!< I2C START Control Bit */
#define I2C_I2CON_STO         (1ul << I2C_I2CON_STO_Pos)   /*!< I2C STOP Control Bit  */
#define I2C_I2CON_SI          (1ul << I2C_I2CON_SI_Pos)	   /*!< I2C Interrupt Flag */
#define I2C_I2CON_AA          (1ul << I2C_I2CON_AA_Pos)	   /*!< Assert Acknowledge Control Bit  */

#define I2C_I2CON_STA_STO_SI        0x38UL /*!< I2CON setting for I2C control bits. It would set STA, STO and SI bits     */
#define I2C_I2CON_STA_STO_SI_AA     0x3CUL /*!< I2CON setting for I2C control bits. It would set STA, STO, SI and AA bits */
#define I2C_I2CON_STA_SI            0x28UL /*!< I2CON setting for I2C control bits. It would set STA and SI bits          */
#define I2C_I2CON_STA_SI_AA         0x2CUL /*!< I2CON setting for I2C control bits. It would set STA, SI and AA bits      */
#define I2C_I2CON_STO_SI            0x18UL /*!< I2CON setting for I2C control bits. It would set STO and SI bits          */
#define I2C_I2CON_STO_SI_AA         0x1CUL /*!< I2CON setting for I2C control bits. It would set STO, SI and AA bits      */
//#define I2C_I2CON_SI                0x08UL /*!< I2CON setting for I2C control bits. It would set SI bit                   */
#define I2C_I2CON_SI_AA             0x0CUL /*!< I2CON setting for I2C control bits. It would set SI and AA bits           */
//#define I2C_I2CON_STA               0x20UL /*!< I2CON setting for I2C control bits. It would set STA bit                  */
//#define I2C_I2CON_AA                0x04UL /*!< I2CON setting for I2C control bits. It would set AA bit                   */


/*---------------------------------------------------------------------------------------------------------*/
/*  I2CTOC constant definitions                                                                             */
/*---------------------------------------------------------------------------------------------------------*/
#define I2C_I2CTOC_ENTI          (1ul << I2C_I2CTOC_ENTI_Pos)   /*!< Time-out Counter Enable/Disable */
#define I2C_I2CTOC_DIV4          (1ul << I2C_I2CTOC_DIV4_Pos)   /*!< Time-out Counter Input Clock Divided by 4*/
#define I2C_I2CTOC_TIF           (1ul << I2C_I2CTOC_TIF_Pos)    /*!< Time-Out Flag */
/*---------------------------------------------------------------------------------------------------------*/
/*  I2CADDR0 constant definitions                                                                             */
/*---------------------------------------------------------------------------------------------------------*/
#define I2C_I2CADDR_GC			 (1ul << 0)					   /*!< General Call Function */

/*---------------------------------------------------------------------------------------------------------*/
/*  I2CWKUPCON constant definitions                                                                             */
/*---------------------------------------------------------------------------------------------------------*/
#define I2C_I2CWKUPCON_WKUPEN          (1ul << I2C_I2CWKUPCON_WKUPEN_Pos)	  /*!< I2C Wake-up Function Enable */

/*---------------------------------------------------------------------------------------------------------*/
/*  I2CWKUPSTS constant definitions                                                                             */
/*---------------------------------------------------------------------------------------------------------*/
#define I2C_I2CWKUPCON_WKUPIF          (1ul << I2C_I2CWKUPSTS_WKUPIF_Pos)	 /*!< Wake-up Interrupt Flag */


#endif

