/******************************************************************************
 * @file     PS2.h
 * @version  V1.0
 * $Revision: 1 $
 * $Date: 14/09/09 4:43p $
 * @brief    NUC123 Series PS2 Driver Header File
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#ifndef __PS2_H__
#define __PS2_H__

#include "NUC123Series.h"

/*---------------------------------------------------------------------------------------------------------*/
/*  PS2CON constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define PS2_PS2CON_DATA_LINE_HIGH       (1UL<<PS2_PS2CON_FPS2DAT_Pos) /*!< If OVERRIDE is set to high, forces PS2DATA line high */
#define PS2_PS2CON_DATA_LINE_LOW        (0UL<<PS2_PS2CON_FPS2DAT_Pos) /*!< If OVERRIDE is set to high, forces PS2DATA line low */

#define PS2_PS2CON_CLK_LINE_HIGH        (1UL<<PS2_PS2CON_FPS2CLK_Pos) /*!< If OVERRIDE is set to high, forces PS2CLK  line high */
#define PS2_PS2CON_CLK_LINE_LOW         (0UL<<PS2_PS2CON_FPS2CLK_Pos) /*!< If OVERRIDE is set to high, forces PS2CLK  line low */

#define PS2_PS2CON_OVERRIDE_EN          (1UL<<PS2_PS2CON_OVERRIDE_Pos) /*!< PS2CLK and PS2DATA pins are controlled by software */
#define PS2_PS2CON_OVERRIDE_DIS         (0UL<<PS2_PS2CON_OVERRIDE_Pos) /*!< PS2CLK and PS2DATA pins are controlled by internal state machine */

#define PS2_PS2CON_CLR_TX_FIFO          (1UL<<PS2_PS2CON_CLRFIFO_Pos) /*!< Clear TX FIFO */

#define PS2_PS2CON_NO_ERR_ACK           (1UL<<PS2_PS2CON_ACK_Pos) /*!< If parity error or stop bit is not received correctly, non acknowledge sent to host at 12th clock */
#define PS2_PS2CON_ALWAYS_ACK           (0UL<<PS2_PS2CON_ACK_Pos) /*!< Always sends acknowledge to host at 12th clock for host to device communication */

#define PS2_PS2CON_RXINT_EN             (1UL<<PS2_PS2CON_RXINTEN_Pos) /*!< Enable data receive complete interrupt */
#define PS2_PS2CON_RXINT_DIS            (0UL<<PS2_PS2CON_RXINTEN_Pos) /*!< Disable data receive complete interrupt */

#define PS2_PS2CON_TXINT_EN             (1UL<<PS2_PS2CON_TXINTEN_Pos) /*!< Enable data transmit complete interrupt */
#define PS2_PS2CON_TXINT_DIS            (0UL<<PS2_PS2CON_TXINTEN_Pos) /*!< Disable data transmit complete interrupt */

#define PS2_PS2CON_ENABLE               (1UL<<PS2_PS2CON_PS2EN_Pos) /*!< Enable PS2 device controller */
#define PS2_PS2CON_DISABLE              (0UL<<PS2_PS2CON_PS2EN_Pos) /*!< Disable PS2 device controller */

/*---------------------------------------------------------------------------------------------------------*/
/*  PS2STATUS constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define PS2_TX_FIFO_EMPTY               (1UL<<PS2_PS2STATUS_TXEMPTY_Pos) /*!< Indicate transmit FIFO empty or not */
#define PS2_RX_BUF_OVERWRITE            (1UL<<PS2_PS2STATUS_RXOVF_Pos) /*!< Indicate if data in PS2RXDATA register is overwritten by new received data */
#define PS2_TRANSMIT_BUSY               (1UL<<PS2_PS2STATUS_TXBUSY_Pos) /*!< Indicate if the PS/2 device is currently sending data */
#define PS2_RECEIVE_BUSY                (1UL<<PS2_PS2STATUS_RXBUSY_Pos) /*!< Indicate if the PS/2 device is currently receiving  data */
#define PS2_RECEIVE_PARITY              (1UL<<PS2_PS2STATUS_RXPARITY_Pos) /*!< It reflects the parity bit for the last received data byte (odd parity) */
#define PS2_FRAME_ERROR                 (1UL<<PS2_PS2STATUS_FRAMERR_Pos) /*!< It reflects the parity bit for the last received data byte (odd parity) */
#define PS2_DATA_PIN_STATUS             (1UL<<PS2_PS2STATUS_PS2DATA_Pos) /*!< It reflects the status of the PS2DATA line after synchronizing and sampling */
#define PS2_CLK_PIN_STATUS              (1UL<<PS2_PS2STATUS_PS2CLK_Pos) /*!< It reflects the status of the PS2CLK line after synchronizing */

/*---------------------------------------------------------------------------------------------------------*/
/*  PS2INTID constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define PS2_TX_INT_STATUS               (1UL<<PS2_PS2INTID_TXINT_Pos) /*!< Transmit interrupt flag */
#define PS2_RX_INT_STATUS               (1UL<<PS2_PS2INTID_RXINT_Pos) /*!< Receive interrupt flag */

/*---------------------------------------------------------------------------------------------------------*/
/* Macro functions                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
/**
 * @brief       Force PS2DATA line high
 *
 * @param       None
 *
 * @return      None
 *
 * @details     If OVERRIDE is set to high, forces PS2DATA line to high level
 */
#define PS2_FORCE_PS2DATA_HIGH()               (PS2->PS2CON = PS2->PS2CON & (~PS2_PS2CON_FPS2DAT_Msk) | PS2_PS2CON_DATA_LINE_HIGH)


/**
 * @brief       Force PS2DATA line low
 *
 * @param       None
 *
 * @return      None
 *
 * @details     If OVERRIDE is set to high, forces PS2DATA line to high low
 */
#define PS2_FORCE_PS2DATA_LOW()                (PS2->PS2CON = PS2->PS2CON & (~PS2_PS2CON_FPS2DAT_Msk) | PS2_PS2CON_DATA_LINE_LOW)


/**
 * @brief       Force PS2CLK line high
 *
 * @param       None
 *
 * @return      None
 *
 * @details     If OVERRIDE is set to high, forces PS2CLK line to high level
 */
#define PS2_FORCE_PS2CLK_HIGH()                (PS2->PS2CON = PS2->PS2CON & (~PS2_PS2CON_FPS2CLK_Msk) | PS2_PS2CON_CLK_LINE_HIGH)


/**
 * @brief       Force PS2CLK line low
 *
 * @param       None
 *
 * @return      None
 *
 * @details     If OVERRIDE is set to high, forces PS2CLK line to high low
 */
#define PS2_FORCE_PS2CLK_LOW()                 (PS2->PS2CON = PS2->PS2CON & (~PS2_PS2CON_FPS2CLK_Msk) | PS2_PS2CON_CLK_LINE_LOW)


/**
 * @brief       Enable software control PS2CLK and PS2DATA pins
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Enable software control PS2CLK and PS2DATA pins
 */
#define PS2_OVERRIDE_ENABLE()                  (PS2->PS2CON = PS2->PS2CON & (~PS2_PS2CON_OVERRIDE_Msk) | PS2_PS2CON_OVERRIDE_EN)


/**
 * @brief       Disable software control PS2CLK and PS2DATA pins
 *
 * @param       None
 *
 * @return      None
 *
 * @details     PS2CLK and PS2DATA pins are controlled by internal state machine
 */
#define PS2_OVERRIDE_DISABLE()                 (PS2->PS2CON = PS2->PS2CON & (~PS2_PS2CON_OVERRIDE_Msk) | PS2_PS2CON_OVERRIDE_DIS)


/**
 * @brief       PS2 clear transmit FIFO
 *
 * @param       None
 *
 * @return      None
 *
 * @details     PS2 clear transmit FIFO
 */
#define PS2_CLR_TX_FIFO()                      (PS2->PS2CON = PS2->PS2CON & (~PS2_PS2CON_CLRFIFO_Msk) | PS2_PS2CON_CLR_TX_FIFO)


/**
 * @brief       PS2 no error acknoledge
 *
 * @param       None
 *
 * @return      None
 *
 * @details     If parity error or stop bit is not received correctly, acknowledge bit will not be sent to host at 12th clock
 */
#define PS2_NO_ERROR_ACK()                     (PS2->PS2CON = PS2->PS2CON & (~PS2_PS2CON_ACK_Msk) | PS2_PS2CON_ALWAYS_ACK)


/**
 * @brief       PS2 always acknoledge
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Always sends acknowledge to host at 12th clock for host to device communication
 */
#define PS2_ALWAYS_ACK()                       (PS2->PS2CON = PS2->PS2CON & (~PS2_PS2CON_ACK_Msk) | PS2_PS2CON_NO_ERR_ACK)


/**
 * @brief       Set PS2 transmit data FIFO depth
 *
 * @param[in]   depth  TX FIFO data depth. It could be 1~16
 *
 * @return      None
 *
 * @details     Set PS2 transmit data FIFO depth, from 1 to 16 bytes
 */
#define PS2_SET_TX_FIFO_DEPTH(depth)           (PS2->PS2CON = PS2->PS2CON & (~PS2_PS2CON_TXFIFO_DEPTH_Msk) | (((depth)-1)<<PS2_PS2CON_TXFIFO_DEPTH_Pos))


/**
 * @brief       PS2 enable data receive complete interrupt
 *
 * @param       None
 *
 * @return      None
 *
 * @details     PS2 enable data receive complete interrupt
 */
#define PS2_RXINT_ENABLE()                     (PS2->PS2CON = PS2->PS2CON & (~PS2_PS2CON_RXINTEN_Msk) | PS2_PS2CON_RXINT_EN)


/**
 * @brief       PS2 disable data receive complete interrupt
 *
 * @param       None
 *
 * @return      None
 *
 * @details     PS2 disable data receive complete interrupt
 */
#define PS2_RXINT_DISABLE()                    (PS2->PS2CON = PS2->PS2CON & (~PS2_PS2CON_RXINTEN_Msk) | PS2_PS2CON_RXINT_DIS)


/**
 * @brief       Get PS2 receive interrupt status
 *
 * @param       None
 *
 * @retval      0  No interrupt
 * @retval      1  Receive interrupt occurs
 *
 * @details     Get PS2 receive interrupt status
 */
#define PS2_GET_RXINT_STATUS()                 ((PS2->PS2INTID & PS2_RX_INT_STATUS)?1:0)


/**
 * @brief       Clear PS2 receive interrupt status
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Clear PS2 receive interrupt status
 */
#define PS2_CLR_RXINT_STATUS()                 (PS2->PS2INTID |= PS2_RX_INT_STATUS)


/**
 * @brief       PS2 enable data transmit complete interrupt
 *
 * @param       None
 *
 * @return      None
 *
 * @details     PS2 enable data transmit complete interrupt
 */
#define PS2_TXINT_ENABLE()                     (PS2->PS2CON = PS2->PS2CON & (~PS2_PS2CON_TXINTEN_Msk) | PS2_PS2CON_TXINT_EN)


/**
 * @brief       PS2 disable data transmit complete interrupt
 *
 * @param       None
 *
 * @return      None
 *
 * @details     PS2 disable data transmit complete interrupt
 */
#define PS2_TXINT_DISABLE()                    (PS2->PS2CON = PS2->PS2CON & (~PS2_PS2CON_TXINTEN_Msk) | PS2_PS2CON_TXINT_DIS)


/**
 * @brief       Get PS2 transmit interrupt status
 *
 * @param       None
 *
 * @retval      0  No interrupt
 * @retval      1  Transmit interrupt occurs
 *
 * @details     Get PS2 transmit interrupt status
 */
#define PS2_GET_TXINT_STATUS()                 ((PS2->PS2INTID & PS2_TX_INT_STATUS)?1:0)


/**
 * @brief       Clear PS2 transmit interrupt status
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Clear PS2 transmit interrupt status
 */
#define PS2_CLR_TXINT_STATUS()                 (PS2->PS2INTID |= PS2_TX_INT_STATUS)


/**
 * @brief       Enable PS2 device controller
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Enable PS2 device controller
 */
#define PS2_ENABLE()                           (PS2->PS2CON = PS2->PS2CON & (~PS2_PS2CON_PS2EN_Msk) | PS2_PS2CON_ENABLE)


/**
 * @brief       Disable PS2 device controller
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Disable PS2 device controller
 */
#define PS2_DISABLE()                          (PS2->PS2CON = PS2->PS2CON & (~PS2_PS2CON_PS2EN_Msk) | PS2_PS2CON_DISABLE)


/**
 * @brief       Set PS2 transmit data register x
 *
 * @param[in]   x     PS2 transmit data register number. It could be 0~3
 * @param[in]   data  Data to be written to PS2 transmit data register x
 *
 * @return      None
 *
 * @details     Set PS2 transmit data register x
 */
#define PS2_SET_PS2TXDATA(x, data)             (PS2->PS2TXDATA[(x)] = (data))


/**
 * @brief       Get PS2 receive data register
 *
 * @param       None
 *
 * @return      Value of PS2 receive data register
 *
 * @details     Get value of PS2 receive data register
 */
#define PS2_GET_PS2RXDATA()                    (PS2->PS2RXDATA)


/**
 * @brief       Get PS2 transmit shift register byte index
 *
 * @param       None
 *
 * @return      Byte index in transmit shift register
 *
 * @details     It indicates which data byte in transmit data shift register. \n
 *              When all data in FIFO is transmitted and it will be cleared to 0
 */
#define PS2_GET_BYTE_INDEX()                   ((PS2->PS2STATUS & PS2_PS2STATUS_BYTEIDX_Msk) >> PS2_PS2STATUS_BYTEIDX_Pos)

#endif
