/**************************************************************************//**
 * @file     SPI.h
 * @version  V2.1
 * $Revision: 1 $
 * $Date: 14/09/09 4:43p $
 * @brief    NUC123 Series SPI Driver Header File
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#ifndef __SPI_H__
#define __SPI_H__

/*---------------------------------------------------------------------------------------------------------*/
/* Include related headers                                                                                 */
/*---------------------------------------------------------------------------------------------------------*/
#include "NUC123Series.h"

/**
 * @brief      Transmit FIFO Buffer Full Indicator
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     0 = Indicates that the transmit FIFO buffer is not full. 
 * @retval     1 = Indicates that the transmit FIFO buffer is full. 
 *
 * @details    Read Transmit FIFO Buffer Full Indicator
 *             Example: SPI_READ_TX_FULL(SPI0)
 */
#define SPI_READ_TX_FULL(port) (((port)->CNTRL & SPI_CNTRL_TX_FULL)>>SPI_CNTRL_TX_FULL_Pos)

/**
 * @brief      Transmit FIFO Buffer empty Indicator
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     0 = Indicates that the transmit FIFO buffer is not empty. 
 * @retval     1 = Indicates that the transmit FIFO buffer is empty. 
 *
 * @details    Read Transmit FIFO Buffer empty Indicator
 *             Example: SPI_READ_TX_EMPTY(SPI0)
 */
#define SPI_READ_TX_EMPTY(port) (((port)->CNTRL & SPI_CNTRL_TX_EMPTY)>>SPI_CNTRL_TX_EMPTY_Pos)

/**
 * @brief      Receive FIFO Buffer Full Indicator
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     0 = Indicates that the Receive FIFO buffer is not full. 
 * @retval     1 = Indicates that the Receive FIFO buffer is full. 
 *
 * @details    Read Receive FIFO Buffer Full Indicator
 *             Example: SPI_READ_RX_FULL(SPI0)
 */
#define SPI_READ_RX_FULL(port) (((port)->CNTRL & SPI_CNTRL_RX_FULL)>>SPI_CNTRL_RX_FULL_Pos)

/**
 * @brief      Receive FIFO Buffer empty Indicator
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     0 = Indicates that the receive FIFO buffer is not empty. 
 * @retval     1 = Indicates that the receive FIFO buffer is empty. 
 *
 * @details    Read Receive FIFO Buffer empty Indicator
 *             Example: SPI_READ_RX_EMPTY(SPI0)
 */
#define SPI_READ_RX_EMPTY(port) (((port)->CNTRL & SPI_CNTRL_RX_EMPTY)>>SPI_CNTRL_RX_EMPTY_Pos)

/**
 * @brief      Variable Clock Enable
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Variable Clock Enable
 *             Example: SPI_ENABLE_VARAIBLE_CLOCK(SPI0)
 */
#define SPI_ENABLE_VARAIBLE_CLOCK(port) (port)->CNTRL |= SPI_CNTRL_VARCLK_EN

/**
 * @brief      Variable Clock Disable
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Variable Clock Disable
 *             Example: SPI_DISABLE_VARAIBLE_CLOCK(SPI0)
 */
#define SPI_DISABLE_VARAIBLE_CLOCK(port) (port)->CNTRL &= ~SPI_CNTRL_VARCLK_EN

/**
 * @brief      Two-Bit Transfer Mode  Enable
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    When TWOB is enabled, the serial transmitted 2-bit data are from SPI_TX1/0, 
 *             and the received 2-bit data input are put in SPI_RX1/0
 *             Example: SPI_ENABLE_TWO_BIT(SPI0)
 */
#define SPI_ENABLE_TWO_BIT(port) (port)->CNTRL |= SPI_CNTRL_TWO_BIT

/**
 * @brief      Two-Bit Transfer Mode  Disable
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Two-Bit Transfer Mode Disable 
 *             Example: SPI_DISABLE_TWO_BIT(SPI0)
 */
#define SPI_DISABLE_TWO_BIT(port) (port)->CNTRL &= ~SPI_CNTRL_TWO_BIT

/**
 * @brief      FIFO  Enable
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    FIFO  Enable 
 * @Notes:     Before enabling FIFO mode, the other related settings should be set in advance
 *             Example: SPI_ENABLE_FIFO(SPI0)
 */
#define SPI_ENABLE_FIFO(port) (port)->CNTRL |= SPI_CNTRL_FIFO

/**
 * @brief      FIFO  Disable
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    FIFO  Disable 
 *             Example: SPI_DISABLE_FIFO(SPI0)
 */
#define SPI_DISABLE_FIFO(port) (port)->CNTRL &= ~SPI_CNTRL_FIFO

/**
 * @brief      Byte Reorder Mode Function Enable
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Byte Reorder Mode Function Enable 
 *             Example: SPI_ENABLE_REORDER(SPI0)
 */
#define SPI_ENABLE_REORDER(port) (port)->CNTRL |= SPI_CNTRL_REORDER

/**
 * @brief      Byte Reorder Mode Function Disable
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Byte Reorder Mode Function Disable. 
 *             Example: SPI_DISABLE_REORDER(SPI0)
 */
#define SPI_DISABLE_REORDER(port) (port)->CNTRL &= ~SPI_CNTRL_REORDER

/**
 * @brief      Set slave mode
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Set slave mode.
 *             Example: SPI_SET_SLAVE(SPI0)
 */
#define SPI_SET_SLAVE(port) (port)->CNTRL |= SPI_CNTRL_SLAVE

 /**
 * @brief      Set master mode
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Set master mode.
 *             Example: SPI_SET_MASTER(SPI0)
 */
#define SPI_SET_MASTER(port) (port)->CNTRL &= ~SPI_CNTRL_SLAVE

 /**
 * @brief      Enable SPI transfer interrupt
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    SPI transfer done Interrupt Enabled
 *             Example: SPI_ENABLE_INTERRUPT(SPI0)
 */
#define SPI_ENABLE_INTERRUPT(port) (port)->CNTRL |= SPI_CNTRL_IE

 /**
 * @brief      Disable SPI transfer interrupt
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    SPI transfer done Interrupt disabled
 *             Example: SPI_DISABLE_INTERRUPT(SPI0)
 */
#define SPI_DISABLE_INTERRUPT(port) (port)->CNTRL &= ~SPI_CNTRL_IE

/**
 * @brief      Transfer Done Interrupt Flag
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     0 = It indicates that the transfer does not finish yet. 
 * @retval     1 = It indicates that the SPI controller has finished one unit transfer. 
 *
 * @details    Transfer Done Interrupt Flag
 *             Example: SPI_READ_IF(SPI0)
 */
#define SPI_READ_IF(port) (((port)->CNTRL & SPI_CNTRL_IF)>>SPI_CNTRL_IF_Pos)

/**
 * @brief     Clear Transfer Done Interrupt Flag
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None  
 *
 * @details    Clear Transfer Done Interrupt Flag
 *             Example: SPI_CLEAR_IF(SPI0)
 */
#define SPI_CLEAR_IF(port) (port)->CNTRL |= SPI_CNTRL_IF


/**
 * @brief      Suspend Interval
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @param[in]  x 
 * @return     None  
 *
 * @details    These four bits provide configurable suspend interval
 *             between two successive transmit/receive transaction in a transfer
 *             Example: SPI_SET_SP_CYCLE(SPI0,0x03)
 */
#define SPI_SET_SP_CYCLE(port,x) (port)->CNTRL = (port)->CNTRL&(~SPI_CNTRL_SP_CYCLE_Msk)| SPI_CNTRL_SP_CYCLE(x)


/**
 * @brief      Set SPI clock idle high
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Set SPI clock idle high.
 *             Example: SPI_SET_SPICLK_IDLE_HIGH(SPI0)
 */
#define SPI_SET_SPICLK_IDLE_HIGH (port) (port)->CNTRL |= SPI_CNTRL_CLKP

/**
 * @brief      Set SPI clock idle low
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Set SPI clock idle low.
 *             Example: SPI_SET_SPICLK_IDLE_LOW(SPI0)
 */
#define SPI_SET_SPICLK_IDLE_LOW (port) (port)->CNTRL &= ~SPI_CNTRL_CLKP

/**
 * @brief      LSB First
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    The LSB, bit 0 of the SPI TX0/1 register is sent first to the SPI data output pin ,
 *             and the first bit received from the SPI data input pin will be put in the LSB position of the RX register. 
 *             Example: SPI_SET_LSB_FIRST(SPI0)
 */
#define SPI_SET_LSB_FIRST (port) (port)->CNTRL |= SPI_CNTRL_LSB

/**
 * @brief      MSB First
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    The MSB, which bit of transmit/receive register depends on the setting of TX_BIT_LEN, 
 *             is transmitted/received first.
 *             Example: SPI_SET_MSB_FIRST(SPI0)
 */
#define SPI_SET_MSB_FIRST (port) (port)->CNTRL &= ~SPI_CNTRL_LSB

/**
 * @brief      Transmit Bit Length
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @param[in]  x 	8~32
 * @return     None 
 *
 * @details    This field specifies how many bits can be transmitted / received in one transaction 
 *             Example: SPI_SET_TX_BIT_LEN(SPI0,8)
 */
#define SPI_SET_TX_BIT_LEN (port,x) (port)->CNTRL = (port)->CNTRL&(~SPI_CNTRL_TX_BIT_LEN_Msk)| SPI_CNTRL_TX_BIT_LEN(x)

/**
 * @brief      Transmit on Negative Edge
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Transmitted data output signal is changed on the falling edge of SPICLK.
 *             Example: SPI_SET_TX_FALLING(SPI0)
 */
#define SPI_SET_TX_FALLING (port) (port)->CNTRL |= SPI_CNTRL_TX_NEG

/**
 * @brief      Transmit on Positive Edge
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Transmitted data output signal is changed on the rising edge of SPICLK.
 *             Example: SPI_SET_TX_RISING(SPI0)
 */
#define SPI_SET_TX_RISING (port) (port)->CNTRL &= ~SPI_CNTRL_TX_NEG

/**
 * @brief      Receive on Negative Edge
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Received data input signal is latched on the falling edge of SPICLK.
 *             Example: SPI_SET_RX_FALLING(SPI0)
 */
#define SPI_SET_RX_FALLING (port) (port)->CNTRL |= SPI_CNTRL_RX_NEG

/**
 * @brief      Receive on Positive Edge
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Received data input signal is latched on the rising edge of SPICLK.
 *             Example: SPI_SET_RX_RISING(SPI0)
 */
#define SPI_SET_RX_RISING (port) (port)->CNTRL &= ~SPI_CNTRL_RX_NEG

/**
 * @brief      Set the SPI GO_BUSY bit of the user-specified SPI port
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Set the SPI GO_BUSY bit of the user-specified SPI port.
 *             Example: SPI_SET_GO(SPI0)
 */
#define SPI_SET_GO(port) ((port)->CNTRL |= SPI_CNTRL_GO_BUSY)

/**
 * @brief      Clear the SPI GO_BUSY bit of the user-specified SPI port
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Clear the SPI GO_BUSY bit of the user-specified SPI port.
 *             Example: SPI_CLR_GO(SPI0)
 */
#define SPI_CLR_GO(port) ((port)->CNTRL &= (~SPI_CNTRL_GO_BUSY))

/**
 * @brief      Get the SPI busy status of the user-specified SPI port
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     0 = Data transfer stopped. 
 * @retval     1 = SPI is transferring. 
 *
 * @details    Get the SPI busy status of the user-specified SPI port
 *             Example: SPI_GET_BUSY_STATUS(SPI0)
 */
#define SPI_GET_BUSY_STATUS(port) ((port)->CNTRL & SPI_CNTRL_GO_BUSY)

/**
 * @brief      Set SPI Clock Divider 
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @param[in]  u8div
 * @return     None 
 *
 * @details    Set SPI Clock Divider 
 *             Example: SPI_SET_DIVIDER(SPI0,0x04)
 */
#define SPI_SET_DIVIDER(port,u8div) ((port)->DIVIDER =(port)->DIVIDER & (~SPI_DIVIDER_DIVIDER_Msk)| u8div)

/**
 * @brief      Set SPI Clock Divider 
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @param[in]  u8div
 * @return     None 
 *
 * @details    Set SPI Clock Divider 
 *             Example: SPI_SET_DIVIDER2(SPI0,0x04)
 */
#define SPI_SET_DIVIDER2(port,u8div) ((port)->DIVIDER =(port)->DIVIDER & (~SPI_DIVIDER_DIVIDER2_Msk)| (u8div<<SPI_DIVIDER_DIVIDER2_Pos))

/**
 * @brief      Set slave select mode
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @param[in]  ssrmode
 *              - SPI_SSR_SLAVE_FALLING_EDGE_TRIGGER  
 *              - SPI_SSR_SLAVE_RISING_EDGE_TRIGGER  
 *              - SPI_SSR_SLAVE_LOW_LEVEL_ACTIVE
 *              - SPI_SSR_SLAVE_HIGH_LEVEL_ACTIVE
 *              - SPI_SSR_HW_AUTO_ACTIVE_LOW
 *              - SPI_SSR_HW_AUTO_ACTIVE_HIGH
 *              - SPI_SSR_SW_SS_PIN_LOW
 *              - SPI_SSR_SW_SS_PIN_HIGH
 * @return     None 
 *
 * @details    Set slave select mode
 *             Example: SPI_SET_SLAVE_SEL_MODE(SPI0,SPI_SSR_SLAVE_HIGH_LEVEL_ACTIVE)
 */
#define SPI_SET_SLAVE_SEL_MODE(port,ssrmode) ((port)->SSR = ssrmode )

/**
 * @brief      Level Trigger Accomplish Flag
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     0 = The transferred bit length of one transaction does not meet the specified requirement. 
 * @retval     1 = The transferred bit length meets the specified requirement which defined in TX_BIT_LEN. 
 *
 * @details    Get the level trigger Accomplish flag of the user-specified SPI port
 *             Example: SPI_GET_SLAVE_LEVEL_TRIG_SUCCESS_FLAG(SPI0)
 */
#define SPI_GET_SLAVE_LEVEL_TRIG_SUCCESS_FLAG(port) (((port)->SSR & SPI_SSR_LTRIG_FLAG_Msk)>>SPI_SSR_LTRIG_FLAG_Pos)


/**
 * @brief      Set SPI slave select low level active 
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Configure the user-specified SPI port as a low-level-active device when it is in slave mode
 *             Example: SPI_SET_SLAVE_SELECT_LOW_LEVEL_ACTIVE(SPI0)
 */
#define SPI_SET_SLAVE_SELECT_LOW_LEVEL_ACTIVE(port) ((port)->SSR = (port)->SSR & (~SPI_SSR_SS_LVL_Msk) | SPI_SSR_SS_LTRIG_Msk)

/**
 * @brief      Set SPI slave select high level active 
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Configure the user-specified SPI port as a high-level-active device when it is in slave mode
 *             Example: SPI_SET_SLAVE_SELECT_HIGH_LEVEL_ACTIVE(SPI0)
 */
#define SPI_SET_SLAVE_SELECT_HIGH_LEVEL_ACTIVE(port) ((port)->SSR |= (SPI_SSR_SS_LTRIG_Msk|SPI_SSR_SS_LVL_Msk))

/**
 * @brief      Set SPI slave select falling-edge-active
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Configure the user-specified SPI port as a falling-edge-active device when it is in slave mode
 *             Example: SPI_SET_SLAVE_SELECT_FALLING_EDGE_ACTIVE(SPI0)
 */
#define SPI_SET_SLAVE_SELECT_FALLING_EDGE_ACTIVE(port) ((port)->SSR &= (~(SPI_SSR_SS_LTRIG_Msk|SPI_SSR_SS_LVL_Msk)))

/**
 * @brief      Set SPI slave select rising-edge-active
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Configure the user-specified SPI port as a rising-edge-active device when it is in slave mode
 *             Example: SPI_SET_SLAVE_SELECT_RISING_EDGE_ACTIVE(SPI0)
 */
#define SPI_SET_SLAVE_SELECT_RISING_EDGE_ACTIVE(port) ((port)->SSR = (port)->SSR & (~SPI_SSR_SS_LTRIG_Msk) | SPI_SSR_SS_LVL_Msk)

/**
 * @brief      Enable the automatic slave select function of the user-specified SPI port
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Enable the automatic slave select function of the user-specified SPI port
 *             Example: SPI_ENABLE_HW_AUTO_SLAVE_SELECT(SPI0)
 */
#define SPI_ENABLE_HW_AUTO_SLAVE_SELECT(port) ((port)->SSR |= SPI_SSR_AUTOSS_Msk)

/**
 * @brief      Disable the automatic slave select function of the user-specified SPI port
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Disable the automatic slave select function of the user-specified SPI port
 *             Example: SPI_DISABLE_HW_AUTO_SLAVE_SELECT(SPI0)
 */
#define SPI_DISABLE_HW_AUTO_SLAVE_SELECT(port) ((port)->SSR &= (~SPI_SSR_AUTOSS_Msk))

/**
 * @brief      Set the slave select control bit to 1
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Set the slave select control bit to 1
 *             Example: SPI_SET_SLAVE_SELECT_CNTRL_BIT(SPI0)
 */
#define SPI_SET_SLAVE_SELECT_CNTRL_BIT(port) ((port)->SSR |= SPI_SSR_SSR_Msk)

/**
 * @brief      Clear the slave select control bit to 0
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Clear the slave select control bit to 0
 *             Example: SPI_CLR_SLAVE_SELECT_CNTRL_BIT(SPI0)
 */
#define SPI_CLR_SLAVE_SELECT_CNTRL_BIT(port) ((port)->SSR &= (~SPI_SSR_SSR_Msk))

/**
 * @brief      Get the SPI Rx0 data
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     RX[0] 
 *
 * @details    Get the SPI Rx0 data
 *             Example: SPI_GET_RX0_DATA(SPI0)
 */
#define SPI_GET_RX0_DATA(port) ((port)->RX[0])

/**
 * @brief      Get the SPI Rx1 data
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     RX[1] 
 *
 * @details    Get the SPI Rx1 data
 *             Example: SPI_GET_RX1_DATA(SPI0)
 */
#define SPI_GET_RX1_DATA(port) ((port)->RX[1])

/**
 * @brief      Write data to the SPI Tx buffer0
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @param[in]  x	  data to the SPI Tx buffer0
 * @return     None 
 *
 * @details    Write data to the SPI Tx buffer0
 *             Example: SPI_WRITE_TX_BUFFER0(SPI0,0x55)
 */
#define SPI_WRITE_TX_BUFFER0(port, x) ((port)->TX[0] = (x))

/**
 * @brief      Write data to the SPI Tx buffer1
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @param[in]  x	  data to the SPI Tx buffer1
 * @return     None 
 *
 * @details    Write data to the SPI Tx buffer1
 *             Example: SPI_WRITE_TX_BUFFER1(SPI0,0x55)
 */
#define SPI_WRITE_TX_BUFFER1(port, x) ((port)->TX[1] = (x))

/**
 * @brief      PDMA Reset
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Reset the PDMA control logic in this SPI controller. This bit will be cleared to 0 automatically
 *             Example: SPI_PDMA_RST(SPI0)
 */
#define SPI_PDMA_RST(port) ((port)->DMA |= SPI_DMA_PDMA_RST	)

/**
 * @brief      Receive DMA Start
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Hardware will clear this bit to 0 automatically after PDMA transfer is done
 *             Example: SPI_RX_DMA_GO(SPI0)
 */
#define SPI_RX_DMA_GO(port) ((port)->DMA |= SPI_DMA_RX_DMA_GO)

/**
 * @brief      Transmit DMA Start
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Hardware will clear this bit to 0 automatically after PDMA transfer is done
 *             Example: SPI_TX_DMA_GO(SPI0)
 */
#define SPI_TX_DMA_GO(port) ((port)->DMA |= SPI_DMA_TX_DMA_GO)

/**
 * @brief      SPI Engine Clock Backward Compatible Option
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    The clock configuration is not backward compatible
 *             Example: SPI_DISABLE_BCN(SPI0)
 */
#define SPI_DISABLE_BCN(port) ((port)->CNTRL2  |= SPI_CNTRL2_BCn)

/**
 * @brief      SPI Engine Clock Backward Compatible Option
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Backward compatible clock configuration
 *             Example: SPI_ENABLE_BCN(SPI0)
 */
#define SPI_ENABLE_BCN(port) ((port)->CNTRL2 &= ~SPI_CNTRL2_BCn)

/**
 * @brief      Slave Select Inactive Interrupt Option
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    As the slave select signal goes to inactive level, the IF bit will be set to 1
 *             Example: SPI_ENABLE_SS_INT(SPI0)
 */
#define SPI_ENABLE_SS_INT(port) ((port)->CNTRL2 |= SPI_CNTRL2_SS_INT_OPT)

/**
 * @brief      Slave Select Inactive Interrupt Option
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    As the slave select signal goes to inactive level, the IF bit will NOT be set to 1
 *             Example: SPI_DISABLE_SS_INT(SPI0)
 */
#define SPI_DISABLE_SS_INT(port) ((port)->CNTRL2 &= ~SPI_CNTRL2_SS_INT_OPT)

/**
 * @brief      Dual  IO Mode
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Dual IO Mode function Enabled
 *             Example: SPI_ENABLE_DUAL_IO(SPI0)
 */
#define SPI_ENABLE_DUAL_IO(port) ((port)->CNTRL2 |= SPI_CNTRL2_DUAL_IO_EN)

/**
 * @brief      Dual  IO Mode
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Dual IO Mode function Disabled
 *             Example: SPI_DISABLE_DUAL_IO(SPI0)
 */
#define SPI_DISABLE_DUAL_IO(port) ((port)->CNTRL2 &= ~SPI_CNTRL2_DUAL_IO_EN)

/**
 * @brief      Dual  IO Direction
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Dual output mode
 *             Example: SPI_SET_DUAL_IO_OUTPUT(SPI0)
 */
#define SPI_SET_DUAL_IO_OUTPUT(port) ((port)->CNTRL2 |= SPI_CNTRL2_DUAL_IO_DIR)

/**
 * @brief      Dual  IO Direction
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Dual input mode
 *             Example: SPI_SET_DUAL_IO_INPUT(SPI0)
 */
#define SPI_SET_DUAL_IO_INPUT(port) ((port)->CNTRL2 &= ~SPI_CNTRL2_DUAL_IO_DIR)

/**
 * @brief      Slave 3-Wire Mode Start Interrupt Status
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     0 = It indicates that the SPI transfer is not active. 
 * @retval     1 = It indicates that the transfer start in slave mode with no slave selec.  
 *
 * @details    Get Slave 3-Wire Mode Start Interrupt Status
 *             Example: SPI_GET_3WIRE_START_INT_FLAG(SPI0)
 */
#define SPI_GET_3WIRE_START_INT_FLAG(port) (((port)->CNTRL2 & SPI_CNTRL2_SLV_START_INTSTS)>>SPI_CNTRL2_SLV_START_INTSTS_Pos)

/**
 * @brief      Clear the SPI 3-wire mode start interrupt flag
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Clear the SPI 3-wire mode start interrupt flag
 *             Example: SPI_CLR_3WIRE_START_INT_FLAG(SPI0)
 */
#define SPI_CLR_3WIRE_START_INT_FLAG(port) ((port)->CNTRL2 |= SPI_CNTRL2_SLV_START_INTSTS  )

/**
 * @brief      Enable the SPI 3-wire mode start interrupt
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Enable the SPI 3-wire mode start interrupt
 *             Example: SPI_ENABLE_3WIRE_START_INT(SPI0)
 */
#define SPI_ENABLE_3WIRE_START_INT(port) ((port)->CNTRL2 |= SPI_CNTRL2_SSTA_INTEN)

/**
 * @brief      Disable the SPI 3-wire mode start interrupt
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Disable the SPI 3-wire mode start interrupt
 *             Example: SPI_DISABLE_3WIRE_START_INT(SPI0)
 */
#define SPI_DISABLE_3WIRE_START_INT(port) ((port)->CNTRL2 &= (~SPI_CNTRL2_SSTA_INTEN))

/**
 * @brief      Abort the SPI transfer when the 3-wire mode is enabled
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Abort the SPI transfer when the 3-wire mode is enabled
 *             Example: SPI_ABORT_3WIRE_TRANSFER(SPI0)
 */
#define SPI_ABORT_3WIRE_TRANSFER(port) ((port)->CNTRL2 |= SPI_CNTRL2_SLV_ABORT)


/**
 * @brief      Enable the SPI 3-wire mode
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Enable the SPI 3-wire mode
 *             Example: SPI_ENABLE_3WIRE_MODE(SPI0)
 */
#define SPI_ENABLE_3WIRE_MODE(port) ((port)->CNTRL2 |= SPI_CNTRL2_NOSLVSEL)

/**
 * @brief      Disable the SPI 3-wire mode
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Disable the SPI 3-wire mode
 *             Example: SPI_DISABLE_3WIRE_MODE(SPI0)
 */
#define SPI_DISABLE_3WIRE_MODE(port) ((port)->CNTRL2 &= (~SPI_CNTRL2_NOSLVSEL))

/**
 * @brief      Transmit FIFO Threshold
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @param[in]  x 	
 * @return     None 
 *
 * @details    If the valid data count of the transmit FIFO buffer is less than or equal to the TX_THRESHOLD setting, the TX_INTSTS bit will be set to 1
 *             Example: SPI_SET_TX_FIFO_THRESHOLD(SPI0,7)
 */
#define SPI_SET_TX_FIFO_THRESHOLD (port,x) (port)->FIFO_CTL = (port)->FIFO_CTL&(~SPI_FIFO_CTL_TX_THRESHOLD_Msk)| SPI_FIFO_TX_THRESHOLD(x)

/**
 * @brief      Received  FIFO Threshold
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @param[in]  x 	
 * @return     None 
 *
 * @details    If the valid data count of the receive FIFO buffer is larger than the RX_THRESHOLD setting, the RX_INTSTS bit will be set to 1
 *             Example: SPI_SET_RX_FIFO_THRESHOLD(SPI0,7)
 */
#define SPI_SET_RX_FIFO_THRESHOLD (port,x) (port)->FIFO_CTL = (port)->FIFO_CTL&(~SPI_FIFO_CTL_RX_THRESHOLD_Msk)| SPI_FIFO_RX_THRESHOLD(x)

/**
 * @brief      Receive FIFO Time-out Interrupt Enable
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Receive FIFO Time-out Interrupt Enable
 *             Example: SPI_ENABLE_TIMEOUT_INT(SPI0)
 */
#define SPI_ENABLE_TIMEOUT_INT(port) ((port)->FIFO_CTL |= SPI_TIMEOUT_INTEN)

/**
 * @brief      Receive FIFO Time-out Interrupt Disable
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Receive FIFO Time-out Interrupt Disable
 *             Example: SPI_DISABLE_TIMEOUT_INT(SPI0)
 */
#define SPI_DISABLE_TIMEOUT_INT(port) ((port)->FIFO_CTL &= ~SPI_TIMEOUT_INTEN)

/**
 * @brief      Receive FIFO Over Run Interrupt Enable
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Receive FIFO Over Run Interrupt Enable
 *             Example: SPI_ENABLE_RXOVER_INT(SPI0)
 */
#define SPI_ENABLE_RXOVER_INT(port) ((port)->FIFO_CTL |= SPI_RXOVER_INTEN)

/**
 * @brief      Receive FIFO Over Run Interrupt Disable
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    Receive FIFO Over Run Interrupt Disable
 *             Example: SPI_DISABLE_RXOVER_INT(SPI0)
 */
#define SPI_DISABLE_RXOVER_INT(port) ((port)->FIFO_CTL &= ~SPI_RXOVER_INTEN)

/**
 * @brief      TX Threshold Interrupt Enable
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    TX Threshold Interrupt Enable
 *             Example: SPI_ENABLE_TX_THRESHOLD_INT(SPI0)
 */
#define SPI_ENABLE_TX_THRESHOLD_INT(port) ((port)->FIFO_CTL |= SPI_TX_INTEN)

/**
 * @brief      TX Threshold Interrupt Disable
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    TX Threshold Interrupt Disable
 *             Example: SPI_DISABLE_TX_THRESHOLD_INT(SPI0)
 */
#define SPI_DISABLE_TX_THRESHOLD_INT(port) ((port)->FIFO_CTL &= ~SPI_TX_INTEN)

/**
 * @brief      RX Threshold Interrupt Enable
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    RX Threshold Interrupt Enable
 *             Example: SPI_ENABLE_RX_THRESHOLD_INT(SPI0)
 */
#define SPI_ENABLE_RX_THRESHOLD_INT(port) ((port)->FIFO_CTL |= SPI_RX_INTEN)

/**
 * @brief      RX Threshold Interrupt Disable
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    RX Threshold Interrupt Disable
 *             Example: SPI_DISABLE_RX_THRESHOLD_INT(SPI0)
 */
#define SPI_DISABLE_RX_THRESHOLD_INT(port) ((port)->FIFO_CTL &= ~SPI_RX_INTEN)

/**
 * @brief      Clear transmit FIFO buffer
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    This bit will be cleared to 0 by hardware after software sets it to 1
 *             Example: SPI_CLEAR_TX_FIFO_BUFFER(SPI0)
 */
#define SPI_CLEAR_TX_FIFO_BUFFER(port) ((port)->FIFO_CTL |= SPI_TX_CLR)

/**
 * @brief      Clear receive FIFO buffer
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @return     None 
 *
 * @details    This bit will be cleared to 0 by hardware after software sets it to 1
 *             Example: SPI_CLEAR_RX_FIFO_BUFFER(SPI0)
 */
#define SPI_CLEAR_RX_FIFO_BUFFER(port) ((port)->FIFO_CTL |= SPI_RX_CLR)

/**
 * @brief      Transmit FIFO Data Count 
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     The valid data count of transmit FIFO buffer
 *
 * @details    Indicates the valid data count of transmit FIFO buffer
 *             Example: SPI_GET_TX_FIFO_COUNT(SPI0)
 */
#define SPI_GET_TX_FIFO_COUNT(port) (((port)->STATUS & SPI_STATUS_TX_FIFO_COUNT_Msk)>>SPI_STATUS_TX_FIFO_COUNT_Pos)

/**
 * @brief      Transmit FIFO Buffer Full Indicator
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     0    Indicates that the transmit FIFO buffer is not full
 * @retval     1    Indicates that the transmit FIFO buffer is full
 *
 * @details    Transmit FIFO Buffer Full Indicator
 *             Example: SPI_GET_TX_FULL(SPI0)
 */
#define SPI_GET_TX_FULL(port) (((port)->STATUS & SPI_STATUS_TX_FULL)>>SPI_STATUS_TX_FULL_Pos)

/**
 * @brief      Transmit FIFO Buffer empty Indicator
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     0    Indicates that the transmit FIFO buffer is not empty
 * @retval     1    Indicates that the transmit FIFO buffer is empty
 *
 * @details    Transmit FIFO Buffer empty Indicator
 *             Example: SPI_GET_TX_EMPTY(SPI0)
 */
#define SPI_GET_TX_EMPTY(port) (((port)->STATUS & SPI_STATUS_TX_EMPTY)>>SPI_STATUS_TX_EMPTY_Pos)

/**
 * @brief      Receive FIFO Buffer Full Indicator
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     0    Indicates that the receive FIFO buffer is not full
 * @retval     1    Indicates that the receive FIFO buffer is full
 *
 * @details    Receive FIFO Buffer Full Indicator
 *             Example: SPI_GET_RX_FULL(SPI0)
 */
#define SPI_GET_RX_FULL(port) (((port)->STATUS & SPI_STATUS_RX_FULL)>>SPI_STATUS_RX_FULL_Pos)

/**
 * @brief      Receive FIFO Buffer empty Indicator
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     0    Indicates that the receive FIFO buffer is not empty
 * @retval     1    Indicates that the receive FIFO buffer is empty
 *
 * @details    Receive FIFO Buffer empty Indicator
 *             Example: SPI_GET_RX_EMPTY(SPI0)
 */
#define SPI_GET_RX_EMPTY(port) (((port)->STATUS & SPI_STATUS_RX_EMPTY)>>SPI_STATUS_RX_EMPTY_Pos)

/**
 * @brief      Time-out Interrupt Flag
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     0    No receive FIFO time-out event
 * @retval     1    receive FIFO time-out event	happen
 *
 * @details    It indicates that the receive FIFO buffer is not empty and there is not be read
 *             over 64 SPI clock period in master mode and over 576 SPI engine clock period in slave mode
 *             Example: SPI_GET_TIMEOUT(SPI0)
 */
#define SPI_GET_TIMEOUT(port) (((port)->STATUS & SPI_STATUS_TIMEOUT)>>SPI_STATUS_TIMEOUT_Pos)

/**
 * @brief      Receive  FIFO Data Count 
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     The valid data count of receive FIFO buffer
 *
 * @details    Indicates the valid data count of receive FIFO buffer
 *             Example: SPI_GET_RX_FIFO_COUNT(SPI0)
 */
#define SPI_GET_RX_FIFO_COUNT(port) (((port)->STATUS & SPI_STATUS_RX_FIFO_COUNT_Msk)>>SPI_STATUS_RX_FIFO_COUNT_Pos)

/**
 * @brief      Transmit FIFO Threshold Interrupt Status
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     0    the valid data count within the transmit FIFO buffer is less than or equal to the setting value of TX_THRESHOLD
 * @retval     1    the valid data count within the transmit FIFO buffer is larger than the setting value of TX_THRESHOLD
 *
 * @details    Transmit FIFO Threshold Interrupt Status
 *             Example: SPI_GET_TX_FIFO_THRESHOLD(SPI0)
 */
#define SPI_GET_TX_FIFO_THRESHOLD(port) (((port)->STATUS & SPI_STATUS_TXINT_STS)>>SPI_STATUS_TX_INTSTS_Pos)

/**
 * @brief      Receive FIFO Overrun Status
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     0   Receive FIFO Not Overrun
 * @retval     1   Receive FIFO Overrun
 *
 * @details    When the receive FIFO buffer is full, the follow-up data will be dropped and this bit will be set to 1
 *             Example: SPI_GET_RX_OVER_RUN(SPI0)
 */
#define SPI_GET_RX_OVER_RUN(port) (((port)->STATUS & SPI_STATUS_RX_OVER_RUN)>>SPI_STATUS_RX_OVERRUN_Pos)

/**
 * @brief      Transmit FIFO Threshold Interrupt Status
 *
 * @param[in]  port    SPI port 
 *                     -SPI0
 *                     -SPI1
 *                     -SPI2
 * @retval     0    the valid data count within the receive FIFO buffer is less than or equal to the setting value of RX_THRESHOLD
 * @retval     1    the valid data count within the receive FIFO buffer is larger than the setting value of RX_THRESHOLD
 *
 * @details    Transmit FIFO Threshold Interrupt Status
 *             Example: SPI_GET_RX_FIFO_THRESHOLD(SPI0)
 */
#define SPI_GET_RX_FIFO_THRESHOLD(port) (((port)->STATUS & SPI_STATUS_RXINT_STS)>>SPI_STATUS_RX_INTSTS_Pos)
/*---------------------------------------------------------------------------------------------------------*/
/* SPI_CNTRL constant definitions                                                                          */
/*---------------------------------------------------------------------------------------------------------*/
#define SPI_CNTRL_TX_FULL                 (1UL << SPI_CNTRL_TX_FULL_Pos)  /*!< Transmit FIFO Buffer Full Indicator  */
#define SPI_CNTRL_TX_EMPTY                (1UL << SPI_CNTRL_TX_EMPTY_Pos) /*!< Transmit FIFO Buffer Empty Indicator  */
#define SPI_CNTRL_RX_FULL                 (1UL << SPI_CNTRL_RX_FULL_Pos)  /*!< Receive FIFO Buffer Full Indicator  */
#define SPI_CNTRL_RX_EMPTY                (1UL << SPI_CNTRL_RX_EMPTY_Pos) /*!< Receive FIFO Buffer Empty Indicator */
#define SPI_CNTRL_VARCLK_EN               (1UL << SPI_CNTRL_VARCLK_EN_Pos)/*!< Variable Clock Enable */
#define SPI_CNTRL_TWO_BIT                 (1UL << SPI_CNTRL_TWOB_Pos)     /*!< Two-Bit Transfer Mode  Enable */ 
#define SPI_CNTRL_FIFO                    (1UL << SPI_CNTRL_FIFO_Pos)     /*!<  FIFO mode Enable */
#define SPI_CNTRL_REORDER                 (1UL << SPI_CNTRL_REORDER_Pos)  /*!< Byte reorder function Enabled */
#define SPI_CNTRL_SLAVE                   (1UL << SPI_CNTRL_SLAVE_Pos)    /*!< Slave mode  */
#define SPI_CNTRL_IE                      (1UL << SPI_CNTRL_IE_Pos)       /*!< Transfer Interrupt Enable */
#define SPI_CNTRL_IF                      (1UL << SPI_CNTRL_IF_Pos)       /*!< Transfer Done Interrupt Flag */
#define SPI_CNTRL_SP_CYCLE(x)             ((x) << SPI_CNTRL_SP_CYCLE_Pos) /*!< Suspend Interval (Master Only) */
#define SPI_CNTRL_CLKP                    (1UL << SPI_CNTRL_CLKP_Pos)     /*!< SPICLK idle high */
#define SPI_CNTRL_LSB                     (1UL << SPI_CNTRL_LSB_Pos)      /*!< LSB First */
#define SPI_CNTRL_TX_BIT_LEN(x)           ((((x)!=32)?(x):0) << SPI_CNTRL_TX_BIT_LEN_Pos)/*!< This field specifies how many bits can be transmitted / received in one transactio */
#define SPI_CNTRL_TX_NEG                  (1UL << SPI_CNTRL_TX_NEG_Pos)   /*!< Transmit on Negative Edge */
#define SPI_CNTRL_RX_NEG                  (1UL << SPI_CNTRL_RX_NEG_Pos)   /*!< Receive on Negative Edge */
#define SPI_CNTRL_GO_BUSY                 (1UL << SPI_CNTRL_GO_BUSY_Pos)  /*!< SPI Transfer Control Bit and Busy Status */
												
/*---------------------------------------------------------------------------------------------------------*/
/* SPI_DIVIDER constant definitions                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define SPI_DIVIDER_DIV(x)  (x)         /*!< SPI_DIVIDER setting for SPI master clock divider. It could be 0~255 */
#define SPI_DIVIDER_DIV2(x) ((x) << 16) /*!< SPI_DIVIDER setting for SPI master clock divider2 of varialbe clock function. It could be 0~255 */
/*---------------------------------------------------------------------------------------------------------*/
/* SPI_SSR constant definitions                                                                            */
/*---------------------------------------------------------------------------------------------------------*/
#define SPI_SSR_SLAVE_FALLING_EDGE_TRIGGER (0x0)  /*!< SPI_SSR setting for falling edge trigger in SPI slave mode */
#define SPI_SSR_SLAVE_RISING_EDGE_TRIGGER  (0x4)  /*!< SPI_SSR setting for rising edge trigger in SPI slave mode */
#define SPI_SSR_SLAVE_LOW_LEVEL_ACTIVE     (0x10) /*!< SPI_SSR setting for low level active in SPI slave mode */
#define SPI_SSR_SLAVE_HIGH_LEVEL_ACTIVE    (0x14) /*!< SPI_SSR setting for high level active in SPI slave mode */
#define SPI_SSR_HW_AUTO_ACTIVE_LOW         (0x9)  /*!< SPI_SSR setting for SPI master SS signal is controled by hardware automatically and active low */
#define SPI_SSR_HW_AUTO_ACTIVE_HIGH        (0xD)  /*!< SPI_SSR setting for SPI master SS signal is controled by hardware automatically active high */
#define SPI_SSR_SW_SS_PIN_LOW              (0x1)  /*!< SPI_SSR setting to force SPI master SS signal to low by software */
#define SPI_SSR_SW_SS_PIN_HIGH             (0x0)  /*!< SPI_SSR setting to force SPI master SS signal to high by software */
/*---------------------------------------------------------------------------------------------------------*/
/* SPI_DMA constant definitions                                                                            */
/*---------------------------------------------------------------------------------------------------------*/
#define SPI_DMA_PDMA_RST                 (1UL << SPI_DMA_PDMA_RST_Pos)	 /*!< PDMA Reset */		  
#define SPI_DMA_RX_DMA_GO                (1UL << SPI_DMA_RX_DMA_GO_Pos)	 /*!< Receive DMA Start */	
#define SPI_DMA_TX_DMA_GO                (1UL << SPI_DMA_TX_DMA_GO_Pos)	 /*!< Transmit DMA Start */	
/*---------------------------------------------------------------------------------------------------------*/
/* SPI_CNTRL2 constant definitions                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#define SPI_CNTRL2_BCn                   (1UL << SPI_CNTRL2_BCn_Pos)		     /*!< SPI Engine Clock Backward Compatible Option */	
#define SPI_CNTRL2_SS_INT_OPT            (1UL << SPI_CNTRL2_SS_INT_OPT_Pos)	     /*!< Slave Select Inactive Interrupt Option */								   
#define SPI_CNTRL2_DUAL_IO_EN            (1UL << SPI_CNTRL2_DUAL_IO_EN_Pos)	     /*!< Dual IO Mode function Enable */
#define SPI_CNTRL2_DUAL_IO_DIR           (1UL << SPI_CNTRL2_DUAL_IO_DIR_Pos)	 /*!< Dual  IO Mode Direction */
#define SPI_CNTRL2_SLV_START_INTSTS      (1UL << SPI_CNTRL2_SLV_START_INTSTS_Pos)/*!< Slave 3-Wire Mode Start Interrupt Status */
#define SPI_CNTRL2_SSTA_INTEN            (1UL << SPI_CNTRL2_SSTA_INTEN_Pos)	     /*!< Slave 3-Wire Mode Start Interrupt Enable */
#define SPI_CNTRL2_SLV_ABORT             (1UL << SPI_CNTRL2_SLV_ABORT_Pos)	     /*!< Slave 3-Wire Mode Abort Control Bit */
#define SPI_CNTRL2_NOSLVSEL              (1UL << SPI_CNTRL2_NOSLVSEL_Pos)	     /*!< No Slave Selected (3-Wire)in Slave mode*/

/*---------------------------------------------------------------------------------------------------------*/
/* SPI_FIFO_CTL constant definitions                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#define SPI_FIFO_TX_THRESHOLD(x)         ((x) << SPI_FIFO_CTL_TX_THRESHOLD_Pos)	/*!<Transmit FIFO Threshold */	
#define SPI_FIFO_RX_THRESHOLD(x)         ((x) << SPI_FIFO_CTL_RX_THRESHOLD_Pos)	/*!< Received FIFO Threshold */	
#define SPI_TIMEOUT_INTEN                (1UL << SPI_FIFO_CTL_TIMEOUT_INTEN_Pos)/*!< Receive FIFO Time-out Interrupt Enable */	
#define SPI_RXOVER_INTEN                 (1UL << SPI_FIFO_CTL_RXOV_INTEN_Pos)	/*!< Receive FIFO Over Run Interrupt Enable */	
#define SPI_TX_INTEN                     (1UL << SPI_FIFO_CTL_TX_INTEN_Pos)		/*!< TX Threshold Interrupt Enable*/	
#define SPI_RX_INTEN                     (1UL << SPI_FIFO_CTL_RX_INTEN_Pos)	    /*!< RX Threshold Interrupt Enable */	
#define SPI_TX_CLR                       (1UL << SPI_FIFO_CTL_TX_CLR_Pos)	    /*!< Clear Transmit FIFO Buffer */	
#define SPI_RX_CLR                       (1UL << SPI_FIFO_CTL_RX_CLR_Pos)		/*!< Clear Receive FIFO Buffer */	

/*---------------------------------------------------------------------------------------------------------*/
/* SPI_STATUS constant definitions                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#define SPI_STATUS_TX_FULL               (1UL << SPI_STATUS_TX_FULL_Pos)	   /*!< Indicates the valid data count of transmit FIFO buffer */
#define SPI_STATUS_TX_EMPTY              (1UL << SPI_STATUS_TX_EMPTY_Pos)	   /*!< Transmit FIFO Buffer Full Indicator  */
#define SPI_STATUS_RX_FULL               (1UL << SPI_STATUS_RX_FULL_Pos)	   /*!< Transmit FIFO Buffer Empty Indicator  */
#define SPI_STATUS_RX_EMPTY              (1UL << SPI_STATUS_RX_EMPTY_Pos)	   /*!< Receive FIFO Buffer Empty Indicator*/
#define SPI_STATUS_TIMEOUT               (1UL << SPI_STATUS_TIMEOUT_Pos)	   /*!< Time-out Interrupt Flag */
#define SPI_STATUS_IF                    (1UL << SPI_STATUS_IF_Pos)			   /*!< SPI Unit Transfer Interrupt Fla*/
#define SPI_STATUS_SLV_START_INTSTS      (1UL << SPI_STATUS_SLV_START_INTSTS_Pos)/*!<Slave Start Interrupt Status */
#define SPI_STATUS_TXINT_STS             (1UL << SPI_STATUS_TXINT_STS_Pos)	   /*!< Transmit FIFO Threshold Interrupt Status  */
#define SPI_STATUS_RX_OVER_RUN           (1UL << SPI_STATUS_RX_OVERRUN_Pos)	   /*!< Receive FIFO Overrun Status*/
#define SPI_STATUS_RXINT_STS             (1UL << SPI_STATUS_RX_INTSTS_Pos)	   /*!< Receive FIFO Threshold Interrupt Status  */



#endif

