/**************************************************************************//**
 * @file     WWDT.h
 * @version  V2.1
 * $Revision: 1 $
 * $Date: 14/09/09 4:43p $
 * @brief    NUC123 Series Watchdog Driver Header File
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#ifndef __WWDT_H__
#define __WWDT_H__

#include "NUC123Series.h"

/**
 * @brief      Clear Window Watchdog Timer counter 
 *
 * @param[in]  None
 *
 * @return     None
 *                                                           
 * @details    Clear Window Watchdog Timer counter.
 *             Example: WWDT_TIMER_RESET()
 */
#define WWDT_TIMER_RESET()                  (WWDT->WWDTRLD = 0x00005AA5)

/**
 * @brief      WWDT Window Compare Register
 *
 * @param[in]  x   
 *
 * @return     None
 *                                                           
 * @details    Set this register to adjust the valid reload window
 *             Example: WWDT_SET_WINCMP(60)
 */
#define WWDT_SET_WINCMP(x)              (WWDT->WWTCR = (WWDT->WWTCR&(~WWDT_WWDTCR_WINCMP_Msk))| WWDT_WWDTCR_WINCMP(x))

/**
 * @brief      WWDT Pre-scale Period Select
 *
 * @param[in]  x   0~15
 *
 * @return     None
 *                                                           
 * @details    WWDT Pre-scale Period Select
 *             Example: WWDT_SET_PERIOD(6)
 */
#define WWDT_SET_PERIOD(x)              (WWDT->WWTCR = (WWDT->WWTCR&(~WWDT_WWDTCR_PERIODSEL_Msk))| WWDT_WWDTCR_PERIODSEL(x))

/**
 * @brief      WWDT Interrupt Enable
 *
 * @param[in]  None
 *
 * @return     None
 *                                                           
 * @details    Set this bit to enable the Watchdog timer interrupt function.
 *             Example: WWDT_ENABLE_INT()
 */
#define WWDT_ENABLE_INT()              (WWDT->WWDTCR |= WWDT_WWDTCR_WWDTIE)

/**
 * @brief      WWDT Interrupt Disable
 *
 * @param[in]  None
 *
 * @return     None
 *                                                           
 * @details    Disable the Watchdog timer interrupt function.
 *             Example: WWDT_DISABLE_INT()
 */
#define WWDT_DISABLE_INT()              (WWDT->WWDTCR &= ~WWDT_WWDTCR_WWDTIE)

/**
 * @brief      WWDT Enable
 *
 * @param[in]  None
 *
 * @return     None
 *                                                           
 * @details    Set this bit to enable the Window Watchdog timer..
 *             Example: WWDT_TIMER_ENABLE()
 */
#define WWDT_TIMER_ENABLE()              (WWDT->WWDTCR |= WWDT_WWDTCR_WWDTEN)

/**
 * @brief      WWDT Disable
 *
 * @param[in]  None
 *
 * @return     None
 *                                                           
 * @details    Set this bit to disable the Window Watchdog timer.
 *             Example: WWDT_TIMER_DISABLE()
 */
#define WWDT_TIMER_DISABLE()              (WWDT->WWDTCR &= ~WWDT_WWDTCR_WWDTEN)

/**
 * @brief      Get WWDT Timeout Reset Flag
 *
 * @param[in]  None
 *
 * @retval     1 = Window Watchdog Timeout reset occurred
 * @retval     0 = Window Watchdog Timeout reset did not occur
 *                                                           
 * @details    Get WWDT Timeout Reset Flag
 *             Example: WWDT_GET_RESET_FLAG()
 *
 */
#define WWDT_GET_RESET_FLAG()      ((((WWDT->WWDTSR) & WWDT_WWDTSR_WWDTRF) == WWDT_WWDTSR_WWDTRF)? 1:0)

/**
 * @brief      Clear WWDT Timeout Reset Flag
 *
 * @param[in]  None
 *
 * @return     None
 *                                                           
 * @details    Clear WWDT Timeout Reset Flag
 *             Example: WDT_CLEAR_RESET_FLAG()
 */
#define WWDT_CLEAR_RESET_FLAG()    (WWDT->WWDTSR |= WWDT_WWDTSR_WWDTRF)

/**
 * @brief      Get WWDT Timeout interrupt Flag
 *
 * @param[in]  None
 *
 * @retval     1 = Window Watchdog Timeout interrupt occurred
 * @retval     0 = Window Watchdog Timeout interrupt did not occur
 *                                                           
 * @details    Get WWDT Timeout interrupt Flag
 *             Example: WWDT_GET_INT_FLAG()
 *
 */
#define WWDT_GET_INT_FLAG()      ((WWDT->WWDTSR) & WWDT_WWDTSR_WWDTIF) 

/**
 * @brief      Clear WWDT Timeout interrupt Flag
 *
 * @param[in]  None
 *
 * @return     None
 *                                                           
 * @details    Clear WWDT Timeout interrupt Flag
 *             Example: WDT_CLEAR_INT_FLAG()
 */
#define WDT_CLEAR_INT_FLAG()    (WWDT->WWDTSR |= WWDT_WWDTSR_WWDTIF)

/**
 * @brief      WWDT Counter Value
 *
 * @param[in]  None
 *
 * @retval     WWDT Counter Value
 *                                                           
 * @details    Get WWDT Counter Value
 *             Example: WWDT_GET_VALUE()
 *
 */
#define WWDT_GET_VALUE()      ((WWDT->WWDTCVR) & 0x3F) 


/*---------------------------------------------------------------------------------------------------------*/
/* WWDT WWTCR Constants Definitions                                                                              */                                                                            
/*---------------------------------------------------------------------------------------------------------*/
#define WWDT_WWDTCR_WINCMP(x)     		((x) << WWDT_WWDTCR_WINCMP_Pos) /*!< WWDT Window Compare Register  */
#define WWDT_WWDTCR_PERIODSEL(x)        ((x) << WWDT_WWDTCR_PERIODSEL_Pos)/*!< WWDT Pre-scale Period Select  */
#define WWDT_WWDTCR_WWDTIE              (1ul << WWDT_WWDTCR_WWDTIE_Pos)	/*!< WWDT Interrupt Enable  */
#define WWDT_WWDTCR_WWDTEN              (1ul << WWDT_WWDTCR_WWDTEN_Pos)/*!< WWDT Enable */
/*---------------------------------------------------------------------------------------------------------*/
/* WWDT WWDTSR Constants Definitions                                                                              */                                                                            
/*---------------------------------------------------------------------------------------------------------*/
#define WWDT_WWDTSR_WWDTRF              (1ul << WWDT_WWDTSR_WWDTRF_Pos)	/*!< WWDT Reset Flag */
#define WWDT_WWDTSR_WWDTIF              (1ul << WWDT_WWDTSR_WWDTIF_Pos)	/*!< WWDT Compare Match Interrupt Flag */


#endif  
