/**
  * COPYRIGHT 2010 Nuvoton Technology Corp.
  ******************************************************************************
  * @file    nuc200_spi.c
  * @author  NuMicro MCU Software Team
  * @version V1.0.0
  * @date    08/11/2010
  * @brief   This file provides all the SPI firmware functions.
  ******************************************************************************
  */ 

//* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include "DrvSYS.h"
#include "DrvSPI.h"
#include "Driver/DrvGPIO.h"

/* Definitons ------------------------------------------------------------------*/
typedef struct
{
    uint8_t bBusy;
    PFN_DRVSPI_CALLBACK pfncallback;
    uint32_t u32userData;
} S_DRVSPI_HANDLE;

static S_DRVSPI_HANDLE g_sSpiHandler[3];
static SPI_TypeDef *SPI_PORT[3];

/**
  * @brief  SPI0 interrupt handler.
  * @param  None.
  * @retval None.
  */
void SPI0_IRQHandler(void)
{
	// write '1' to clear SPI0 interrupt flag
	SPI0->STATUS_BITS.INTSTS = 1;
	
	if(g_sSpiHandler[0].pfncallback != NULL)
		g_sSpiHandler[0].pfncallback(g_sSpiHandler[0].u32userData);		
}

/**
  * @brief  SPI1 interrupt handler.
  * @param  None.
  * @retval None.
  */
void SPI1_IRQHandler(void)
{
	// write '1' to clear SPI1 interrupt flag
	SPI1->STATUS_BITS.INTSTS = 1;
	
	if(g_sSpiHandler[1].pfncallback != NULL)
		g_sSpiHandler[1].pfncallback(g_sSpiHandler[1].u32userData);
}

/**
  * @brief  SPI2 interrupt handler.
  * @param  None.
  * @retval None.
  */
void SPI2_IRQHandler(void)
{
	// write '1' to clear SPI2 interrupt flag
	SPI2->STATUS_BITS.INTSTS = 1;
	
	if(g_sSpiHandler[2].pfncallback != NULL)
		g_sSpiHandler[2].pfncallback(g_sSpiHandler[2].u32userData);
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function:	DrvGPIO_InitSPI0																		   */
/*																										   */
/* Parameter:																							   */
/*				 SPIFun	 -[in] the struct parameter	to SPI pin function									   */
/*						   include of																	   */
/*							   clk - E_SPI0_CLK_GPC1 / E_SPI0_CLK_GPE2									   */
/*							   ss0 - E_SPI0_SS0_GPC0 / E_SPI0_SS0_GPE1 / E_SPI0_SS0_NONE				   */
/*							   ss1 - 1 is ON (select GPB10); 0 is OFF (do NOT use it pin)				   */
/*							   miso0 - E_SPI0_MISO0_GPB11 /	E_SPI0_MISO0_GPC2 /	E_SPI0_MISO0_GPE3		   */
/*							   mosi0 - E_SPI0_MOSI0_GPB10 /	E_SPI0_MOSI0_GPC3 /	E_SPI0_MOSI0_GPE4		   */
/*							   miso1 - 1 is	ON (select GPC4); 0	is OFF (do NOT use it pin)				   */
/*							   mosi1 - 1 is	ON (select GPC5); 0	is OFF (do NOT use it pin)				   */
/* Returns:																								   */
/*				E_SUCCESS										Operation successful					   */
/*				E_DRVGPIO_ARGUMENT								Incorrect argument						   */
/* Description:																							   */
/*				Initialize the specified function and configure	the	relative pins						   */
/*				for	specified function used.															   */
/*---------------------------------------------------------------------------------------------------------*/
int32_t	DrvGPIO_InitSPI0(STR_GPIO_SPI0_T *SPIFun)
{
	switch (SPIFun->clk)
	{
		case E_SPI0_CLK_GPC1:
		{
			GCR->PC_L_MFP_BITS.PC1 = 0x1;
		}break;

		case E_SPI0_CLK_GPE2:
		{
			GCR->PE_L_MFP_BITS.PE2 = 0x6;
		}break;

		default:
			return E_DRVGPIO_ARGUMENT;
	}

	switch (SPIFun->ss0)
	{
		case E_SPI0_SS0_GPC0:
		{
			GCR->PC_L_MFP_BITS.PC0 = 0x1;
		}break;

		case E_SPI0_SS0_GPE1:
		{
			GCR->PE_L_MFP_BITS.PE1 = 0x6;
		}break;

		case E_SPI0_SS0_NONE:
			break;

		default:
			return E_DRVGPIO_ARGUMENT;
	}

	if (SPIFun->ss1	== 1)	/* GPB10 */
		GCR->PB_H_MFP_BITS.PB10	= 0x1;

	switch (SPIFun->miso0)
	{
		case E_SPI0_MISO0_GPB11:
		{
			GCR->PB_H_MFP_BITS.PB11	= 0x5;
		}break;

		case E_SPI0_MISO0_GPC2:
		{
			GCR->PC_L_MFP_BITS.PC2 = 0x1;
		}break;

		case E_SPI0_MISO0_GPE3:
		{
			GCR->PE_L_MFP_BITS.PE3 = 0x6;
		}break;

		default:
			return E_DRVGPIO_ARGUMENT;
	}

	switch (SPIFun->mosi0)
	{
		case E_SPI0_MOSI0_GPB10:
		{
			GCR->PB_H_MFP_BITS.PB10	= 0x5;
		}break;

		case E_SPI0_MOSI0_GPC3:
		{
			GCR->PC_L_MFP_BITS.PC3 = 0x1;
		}break;

		case E_SPI0_MOSI0_GPE4:
		{
			GCR->PE_L_MFP_BITS.PE4 = 0x6;
		}break;

		default:
			return E_DRVGPIO_ARGUMENT;
	}

	if (SPIFun->miso1 == 1)	/* GPC4	*/
		GCR->PC_L_MFP_BITS.PC4 = 0x1;

	if (SPIFun->mosi1 == 1)	/* GPC5	*/
		GCR->PC_L_MFP_BITS.PC5 = 0x1;

	return E_SUCCESS;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function:	DrvGPIO_InitSPI1																		   */
/*																										   */
/* Parameter:																							   */
/*				 SPIFun	 -[in] the struct parameter	to SPI pin function									   */
/*						   include of																	   */
/*							   clk - E_SPI1_CLK_GPB2 / E_SPI1_CLK_GPC9									   */
/*							   ss0 - E_SPI1_SS0_GPB3 / E_SPI1_SS0_GPC8 / E_SPI1_SS0_NONE				   */
/*							   ss1 - 1 is ON (select GPB9);	0 is OFF (do NOT use it	pin)				   */
/*							   miso0 - E_SPI1_MISO0_GPB1 / E_SPI1_MISO0_GPC10							   */
/*							   mosi0 - E_SPI1_MOSI0_GPB0 / E_SPI1_MOSI0_GPC11							   */
/*							   miso1 - 1 is	ON (select GPC12); 0 is	OFF	(do	NOT	use	it pin)				   */
/*							   mosi1 - 1 is	ON (select GPC13); 0 is	OFF	(do	NOT	use	it pin)				   */
/* Returns:																								   */
/*				E_SUCCESS										Operation successful					   */
/*				E_DRVGPIO_ARGUMENT								Incorrect argument						   */
/* Description:																							   */
/*				Initialize the specified function and configure	the	relative pins						   */
/*				for	specified function used.															   */
/*---------------------------------------------------------------------------------------------------------*/
int32_t	DrvGPIO_InitSPI1(STR_GPIO_SPI1_T *SPIFun)
{
	switch (SPIFun->clk)
	{
		case E_SPI1_CLK_GPB2:
		{
			GCR->PB_L_MFP_BITS.PB2 = 0x3;
		}break;

		case E_SPI1_CLK_GPC9:
		{
			GCR->PC_H_MFP_BITS.PC9 = 0x1;
		}break;

		default:
			return E_DRVGPIO_ARGUMENT;
	}

	switch (SPIFun->ss0)
	{
		case E_SPI1_SS0_GPB3:
		{
			GCR->PB_L_MFP_BITS.PB3 = 0x3;
		}break;

		case E_SPI1_SS0_GPC8:
		{
			GCR->PC_H_MFP_BITS.PC8 = 0x1;
		}break;

		case E_SPI1_SS0_NONE:
			break;

		default:
			return E_DRVGPIO_ARGUMENT;
	}

	if (SPIFun->ss1	== 1)	/* GPB9	*/
		GCR->PB_H_MFP_BITS.PB9 = 0x1;

	switch (SPIFun->miso0)
	{
		case E_SPI1_MISO0_GPB1:
		{
			GCR->PB_L_MFP_BITS.PB1 = 0x3;
		}break;

		case E_SPI1_MISO0_GPC10:
		{
			GCR->PC_H_MFP_BITS.PC10	= 0x1;
		}break;

		default:
			return E_DRVGPIO_ARGUMENT;
	}

	switch (SPIFun->mosi0)
	{
		case E_SPI1_MOSI0_GPB0:
		{
			GCR->PB_L_MFP_BITS.PB0 = 0x3;
		}break;

		case E_SPI1_MOSI0_GPC11:
		{
			GCR->PC_H_MFP_BITS.PC11	= 0x1;
		}break;

		default:
			return E_DRVGPIO_ARGUMENT;
	}

	if (SPIFun->miso1 == 1)	/* GPC12 */
		GCR->PC_H_MFP_BITS.PC12	= 0x1;

	if (SPIFun->mosi1 == 1)	/* GPC13 */
		GCR->PC_H_MFP_BITS.PC13	= 0x1;

	return E_SUCCESS;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function:	DrvGPIO_InitSPI2																		   */
/*																										   */
/* Parameter:																							   */
/*				 SPIFun	 -[in] the struct parameter	to SPI pin function									   */
/*						   include of																	   */
/*							   clk - E_SPI2_CLK_GPA9 / E_SPI2_CLK_GPB5 / E_SPI2_CLK_GPD1				   */
/*							   ss0 - E_SPI2_SS0_GPA8 / E_SPI2_SS0_GPB4 / E_SPI2_SS0_GPD0 / E_SPI2_SS0_NONE */
/*							   ss1 - 1 is ON (select GPB14); 0 is OFF (do NOT use it pin)				   */
/*							   miso0 - E_SPI2_MISO0_GPA10 /	E_SPI2_MISO0_GPB6 /	E_SPI2_MISO0_GPD2		   */
/*							   mosi0 - E_SPI2_MOSI0_GPA11 /	E_SPI2_MOSI0_GPB7 /	E_SPI2_MOSI0_GPD3		   */
/*							   miso1 - 1 is	ON (select GPD4); 0	is OFF (do NOT use it pin)				   */
/*							   mosi1 - 1 is	ON (select GPD5); 0	is OFF (do NOT use it pin)				   */
/* Returns:																								   */
/*				E_SUCCESS										Operation successful					   */
/*				E_DRVGPIO_ARGUMENT								Incorrect argument						   */
/* Description:																							   */
/*				Initialize the specified function and configure	the	relative pins						   */
/*				for	specified function used.															   */
/*---------------------------------------------------------------------------------------------------------*/
int32_t	DrvGPIO_InitSPI2(STR_GPIO_SPI2_T *SPIFun)
{
	switch (SPIFun->clk)
	{
		case E_SPI2_CLK_GPA9:
		{
			GCR->PA_H_MFP_BITS.PA9 = 0x4;
		}break;

		case E_SPI2_CLK_GPB5:
		{
			GCR->PB_L_MFP_BITS.PB5 = 0x4;
		}break;

		case E_SPI2_CLK_GPD1:
		{
			GCR->PD_L_MFP_BITS.PD1 = 0x3;
		}break;

		default:
			return E_DRVGPIO_ARGUMENT;
	}

	switch (SPIFun->ss0)
	{
		case E_SPI2_SS0_GPA8:
		{
			GCR->PA_H_MFP_BITS.PA8 = 0x4;
		}break;

		case E_SPI2_SS0_GPB4:
		{
			GCR->PB_L_MFP_BITS.PB4 = 0x4;
		}break;

		case E_SPI2_SS0_GPD0:
		{
			GCR->PD_L_MFP_BITS.PD0 = 0x3;
		}break;

		case E_SPI2_SS0_NONE:
			break;

		default:
			return E_DRVGPIO_ARGUMENT;
	}

	if (SPIFun->ss1	== 1)	/* GPB14 */
		GCR->PB_H_MFP_BITS.PB14	= 0x4;

	switch (SPIFun->miso0)
	{
		case E_SPI2_MISO0_GPA10:
		{
			GCR->PA_H_MFP_BITS.PA10	= 0x4;
		}break;

		case E_SPI2_MISO0_GPB6:
		{
			GCR->PB_L_MFP_BITS.PB6 = 0x4;
		}break;

		case E_SPI2_MISO0_GPD2:
		{
			GCR->PD_L_MFP_BITS.PD2 = 0x3;
		}break;

		default:
			return E_DRVGPIO_ARGUMENT;
	}

	switch (SPIFun->mosi0)
	{
		case E_SPI2_MOSI0_GPA11:
		{
			GCR->PA_H_MFP_BITS.PA11	= 0x4;
		}break;

		case E_SPI2_MOSI0_GPB7:
		{
			GCR->PB_L_MFP_BITS.PB7 = 0x4;
		}break;

		case E_SPI2_MOSI0_GPD3:
		{
			GCR->PD_L_MFP_BITS.PD3 = 0x3;
		}break;

		default:
			return E_DRVGPIO_ARGUMENT;
	}

	if (SPIFun->miso1 == 1)	/* GPD4	*/
		GCR->PD_L_MFP_BITS.PD4 = 0x3;

	if (SPIFun->mosi1 == 1)	/* GPD5	*/
		GCR->PD_L_MFP_BITS.PD5 = 0x3;

	return E_SUCCESS;
}


/**
  * @brief  Initialize SPI argument.
  * @param  None.
  * @retval None.
  */
  
void DrvSPI_Init(void)
{
	memset(g_sSpiHandler, 0, sizeof(g_sSpiHandler));
	SPI_PORT[0] = SPI0;
	SPI_PORT[1] = SPI1;
	SPI_PORT[2] = SPI2;
}

/**
  * @brief  Configure the operation mode, transfer type and bit length of a 
  *         transaction of the specified SPI port.
  * @param  eSpiPort: Specify the SPI port.
  *			eMode: Specify the operation mode (SPI_MASTER/SPI_SLAVE).
  *			eType: Specify the transfer type (SPI_TYPE0 ~ SPI_TYPE2).
  *			i32BitLength: Specify the bit length in a transaction (1~32)
  * @retval SPI_ERR_INIT: The specified SPI port has been opened before.
  *			SPI_ERR_BUSY: The specified SPI port is in busy status.
  *			SPI_ERR_BIT_LENGTH: The specified bit length is out of range.
  *			E_SUCCESS: Success.
  */
  
/*
SPI_TYPE0:          
             _________________________________
    CS    --|          Active state           |---
               _   _   _   _   _   _   _   _  
    CLK   ____| |_| |_| |_| |_| |_| |_| |_| |_____
              
    Tx    --| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |-----
          
    Rx    --| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |-----


SPI_TYPE1:          
             _________________________________
    CS    --|          Active state           |---
               _   _   _   _   _   _   _   _  
    CLK   ____| |_| |_| |_| |_| |_| |_| |_| |_____
              
    Tx    ----| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |---
          
    Rx    ----| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |---


SPI_TYPE2:
             _________________________________
    CS    --|          Active state           |---
           ___   _   _   _   _   _   _   _   ______ 
    CLK       |_| |_| |_| |_| |_| |_| |_| |_|  
              
    Tx    ----| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |----
          
    Rx    ----| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |----


SPI_TYPE3:
             _________________________________
    CS    --|          Active state           |---
           ___   _   _   _   _   _   _   _   ______ 
    CLK       |_| |_| |_| |_| |_| |_| |_| |_|  
              
    Tx    --| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |------
          
    Rx    --| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |------
*/
int32_t DrvSPI_Open(E_DRVSPI_PORT eSpiPort, E_DRVSPI_MODE eMode, E_DRVSPI_TRANS_TYPE eType, int32_t i32BitLength)
{
	int32_t i32TimeOut;
	
	if(eSpiPort == eDRVSPI_PORT0)
	{
		//enable SPI0 clock
		CLK->APBCLK_BITS.SPI0_EN        =1;
		
		//SPI0 reset
		GCR->IPRST_CTL2_BITS.SPI0_RST	=1;
		GCR->IPRST_CTL2_BITS.SPI0_RST	=0;
		
		//setup multi-function pin for SPI0		
		//GCR->PC_L_MFP_BITS.PC0			=0x01;
		//GCR->PC_L_MFP_BITS.PC1			=0x01;
		//GCR->PC_L_MFP_BITS.PC2			=0x01;
		//GCR->PC_L_MFP_BITS.PC3			=0x01;
		//GCR->PB_H_MFP_BITS.PB10			=0x01;
	}
	else if(eSpiPort == eDRVSPI_PORT1)
	{
		//enable SPI1 clock
		CLK->APBCLK_BITS.SPI1_EN        =1;
		
		//SPI1 reset
		GCR->IPRST_CTL2_BITS.SPI1_RST	=1;
		GCR->IPRST_CTL2_BITS.SPI1_RST	=0;
		
		//setup multi-function pin for SPI1
		//GCR->PC_H_MFP_BITS.PC8			=0x01;
		//GCR->PC_H_MFP_BITS.PC9			=0x01;
		//GCR->PC_H_MFP_BITS.PC10			=0x01;
		//GCR->PC_H_MFP_BITS.PC11			=0x01;
		//GCR->PB_H_MFP_BITS.PB9			=0x01;
	}
	else if(eSpiPort == eDRVSPI_PORT2)
	{
		//enable SPI1 clock
		CLK->APBCLK_BITS.SPI2_EN        =1;
		
		//SPI1 reset
		GCR->IPRST_CTL2_BITS.SPI2_RST	=1;
		GCR->IPRST_CTL2_BITS.SPI2_RST	=0;
		
		//setup multi-function pin for SPI1
		//GCR->PD_L_MFP_BITS.PD0			=0x03;
		//GCR->PD_L_MFP_BITS.PD1			=0x03;
		//GCR->PD_L_MFP_BITS.PD2			=0x03;
		//GCR->PD_L_MFP_BITS.PD3			=0x03;
		//GCR->PB_H_MFP_BITS.PB14			=0x04;
	}
	else	
		return 0;
		
	if(g_sSpiHandler[eSpiPort].bBusy)
		return E_DRVSPI_ERR_INIT;
		
	/* Check busy*/
	i32TimeOut = 0x10000;
	while(SPI_PORT[eSpiPort]->CTL_BITS.GO_BUSY == 1)
	{
		if(i32TimeOut-- <= 0)
			return E_DRVSPI_ERR_BUSY;
	}
	
	g_sSpiHandler[eSpiPort].bBusy = TRUE;
	g_sSpiHandler[eSpiPort].pfncallback = NULL;
	g_sSpiHandler[eSpiPort].u32userData = 0;
	
	/* Bit length 1 ~ 32 */
	if((i32BitLength <= 0) || (i32BitLength > 32))
		return E_DRVSPI_ERR_BIT_LENGTH;
	
	/* "i32BitLength = 0" means 32 bits */
	if(i32BitLength == 32)
		i32BitLength = 0;
		
	SPI_PORT[eSpiPort]->CTL_BITS.TX_BIT_LEN = i32BitLength;
	
	if(eMode == eDRVSPI_SLAVE)
		SPI_PORT[eSpiPort]->CTL_BITS.SLAVE = 1;
	else
		SPI_PORT[eSpiPort]->CTL_BITS.SLAVE = 0;

	/* Default to automatic slave select and it is low active */
	SPI_PORT[eSpiPort]->SSR_BITS.AUTOSS = 1;
	
	/* Transition types */
	if(eType==eDRVSPI_TYPE0)
	{
		SPI_PORT[eSpiPort]->CTL_BITS.CLKP = 0;
		SPI_PORT[eSpiPort]->CTL_BITS.TX_NEG = 1;
		SPI_PORT[eSpiPort]->CTL_BITS.RX_NEG = 0;
	}
	else if(eType==eDRVSPI_TYPE1)
	{
		SPI_PORT[eSpiPort]->CTL_BITS.CLKP = 0;
		SPI_PORT[eSpiPort]->CTL_BITS.TX_NEG = 0;
		SPI_PORT[eSpiPort]->CTL_BITS.RX_NEG = 1;
	}	
	else if(eType==eDRVSPI_TYPE2)
	{
		SPI_PORT[eSpiPort]->CTL_BITS.CLKP = 1;
		SPI_PORT[eSpiPort]->CTL_BITS.TX_NEG = 1;
		SPI_PORT[eSpiPort]->CTL_BITS.RX_NEG = 0;
	}
	else
	{
		SPI_PORT[eSpiPort]->CTL_BITS.CLKP = 1;
		SPI_PORT[eSpiPort]->CTL_BITS.TX_NEG = 0;
		SPI_PORT[eSpiPort]->CTL_BITS.RX_NEG = 1;
	}	

    return E_SUCCESS;
}

/**
  * @brief  Close the specified SPI module and disable the SPI interrupt.
  * @param  eSpiPort: Specify the SPI port.
  * @retval None.
  */
  
void DrvSPI_Close(E_DRVSPI_PORT eSpiPort)
{
	int32_t i32TimeOut;
	
	g_sSpiHandler[eSpiPort].bBusy = FALSE;
	g_sSpiHandler[eSpiPort].pfncallback = NULL;
	g_sSpiHandler[eSpiPort].u32userData = 0;

	/* Wait SPIMS Busy */
	i32TimeOut = 0x10000;
	while(SPI_PORT[eSpiPort]->CTL_BITS.GO_BUSY == 1)
	{
		if(i32TimeOut-- <= 0)
			break;
	}
   
   if(eSpiPort == eDRVSPI_PORT0)
	{
		NVIC_DisableIRQ(SPI0_IRQn);
		GCR->IPRST_CTL2_BITS.SPI0_RST	=1;
		GCR->IPRST_CTL2_BITS.SPI0_RST	=0;
		CLK->APBCLK_BITS.SPI0_EN        =0;
	}
	else if(eSpiPort == eDRVSPI_PORT1)
	{
		NVIC_DisableIRQ(SPI1_IRQn);
		GCR->IPRST_CTL2_BITS.SPI1_RST	=1;
		GCR->IPRST_CTL2_BITS.SPI1_RST	=0;
		CLK->APBCLK_BITS.SPI1_EN        =0;
	}
	else if(eSpiPort == eDRVSPI_PORT2)
	{
		NVIC_DisableIRQ(SPI2_IRQn);
		GCR->IPRST_CTL2_BITS.SPI2_RST	=1;
		GCR->IPRST_CTL2_BITS.SPI2_RST	=0;
		CLK->APBCLK_BITS.SPI2_EN        =0;
	}      
}

/**
  * @brief  Set 2-bit serial data I/O mode.
  *			When enable 2-bit serial data I/O mode, the Tx_NUM must be configure as 0x00 (one transaction in one transfer.)
  * @param  eSpiPort: Specify the SPI port.
  *			bEnable: Enable (TRUE) / Disable (FALSE)
  * @retval None.
  */
void DrvSPI_Set2BitTransferMode(E_DRVSPI_PORT eSpiPort, uint8_t bEnable)
{
#if(0)
	if(bEnable)
	{
		SPI_PORT[eSpiPort]->CTL_BITS.TWOB = 1;
		SPI_PORT[eSpiPort]->CTL_BITS.TX_NUM = 0;
		if(eSpiPort == eDRVSPI_PORT0)
		{
			GCR->PC_L_MFP_BITS.PC4			=0x01;
			GCR->PC_L_MFP_BITS.PC5			=0x01;
		}
		else if(eSpiPort == eDRVSPI_PORT1)
		{
			GCR->PC_H_MFP_BITS.PC12			=0x01;
			GCR->PC_H_MFP_BITS.PC13			=0x01;
		}
		else if(eSpiPort == eDRVSPI_PORT2)
		{
			GCR->PD_L_MFP_BITS.PD4			=0x03;
			GCR->PD_L_MFP_BITS.PD5			=0x03;		
		}		
	}
	else
		SPI_PORT[eSpiPort]->CTL_BITS.TWOB = 0;
#endif	
}

/**
  * @brief  Dertermine to transfer data with LSB first or MSB first
  * @param  eSpiPort: Specify the SPI port.
  *			eEndian:  Specify LSB first or MSB first
  * @retval None.
  */
void DrvSPI_SetEndian(E_DRVSPI_PORT eSpiPort, E_DRVSPI_ENDIAN eEndian)
{

	if(eEndian == eDRVSPI_LSB_FIRST)
	{
		SPI_PORT[eSpiPort]->CTL_BITS.LSB = 1;
	}
	else
		SPI_PORT[eSpiPort]->CTL_BITS.LSB = 0;

}

/**
  * @brief  Set the bit length of SPI transfer.
  * @param  eSpiPort: Specify the SPI port.
  *			i32BitLength: Specify the bit length (1~32 bits).
  * @retval E_SUCCESS: Success.
  *			E_DRVSPI_ERR_BIT_LENGTH: The bit length is out of range.
  */
int32_t DrvSPI_SetBitLength(E_DRVSPI_PORT eSpiPort, int32_t i32BitLength)
{

    if((i32BitLength < 1) || (i32BitLength > 32))
    {
        return E_DRVSPI_ERR_BIT_LENGTH;
    }
    if(i32BitLength == 32)
        i32BitLength = 0;

	SPI_PORT[eSpiPort]->CTL_BITS.TX_BIT_LEN = i32BitLength;
	
    return E_SUCCESS;
}

/**
  * @brief  Enable/disable Byte Reorder function.
  *			The Byte Reorder function is supported only in 16-, 24- and 32-bit transaction mode.
  * @param  eSpiPort: Specify the SPI port.
  *			eReorder: Specify the reorder type
  *						  eDRVSPI_BYTE_REORDER_SUSPEND_DISABLE: both Byte Reorder function and Byte Suspend  
  *                                                             function are disabled.                       
  *                       eDRVSPI_BYTE_REORDER_SUSPEND        : both Byte Reorder function and Byte Suspend  
  *                                                             function are enabled.                        
  *                       eDRVSPI_BYTE_REORDER                : only enable the Byte Reorder function.       
  *                       eDRVSPI_BYTE_SUSPEND                : only enable the Byte Suspend function.       
  *                       The Byte Suspend function is only available in 32-bit transaction.
  * @retval E_SUCCESS: Success.
  *         E_DRVSPI_ERR_BIT_LENGTH: The bit length is not 16-, 24- or 32-bit. 
  */
  
int32_t DrvSPI_SetByteReorder(E_DRVSPI_PORT eSpiPort, E_DRVSPI_BYTE_REORDER eOption)
{
	if( (SPI_PORT[eSpiPort]->CTL_BITS.TX_BIT_LEN) % 8 )
		return E_DRVSPI_ERR_BIT_LENGTH;

	SPI_PORT[eSpiPort]->CTL_BITS.REORDER = eOption;
	
	return E_SUCCESS;
}

/**
  * @brief  Set the number of clock cycle of the suspend interval.  
  * @param  eSpiPort: Specify the SPI port.
  *			i32Interval: In burst transfer mode, this value specified the delay clocks between successive     
  *                      transactions. If the Byte Suspend function is enabled, it specified the delay clocks 
  *                      among each byte. It could be 2~17. 
  * @retval  E_DRVSPI_ERR_SUSPEND_INTERVAL: The suspend interval setting is out of range.
  *          E_SUCCESS: Success.   
  */
int32_t DrvSPI_SetSuspendCycle(E_DRVSPI_PORT eSpiPort, int32_t i32Interval)
{
    if((i32Interval < 2) || (i32Interval > 17))
	{
		return E_DRVSPI_ERR_SUSPEND_INTERVAL;
	}

	SPI_PORT[eSpiPort]->CTL_BITS.SP_CYCLE = i32Interval-2;

	return E_SUCCESS;
}

/**
  * @brief  Set the trigger mode of slave select pin.  
  * @param  eSpiPort: Specify the SPI port.
  *			eSSTriggerMode: Specify the trigger mode. (SPI_EDGE_TRIGGER or SPI_LEVEL_TRIGGER).
  * @retval None.  
  */
void DrvSPI_SetTriggerMode(E_DRVSPI_PORT eSpiPort, E_DRVSPI_SSLTRIG eSSTriggerMode)
{
    SPI_PORT[eSpiPort]->SSR_BITS.SS_LTRIG = eSSTriggerMode;
}

/**
  * @brief  Set the trigger mode of slave select pin.  
  * @param  eSpiPort: Specify the SPI port.
  *			eSSActType: Select the active type of slave select pin. 
  *						SPI_ACTIVE_LOW_FALLING: Slave select pin is active low in level-trigger mode
  *													or falling-edge trigger in edge-trigger mode.
  *						SPI_ACTIVE_HIGH_RISING: Slave select pin is active high in level-trigger mode
  *													or rising-edge trigger in edge-trigger mode.
  * @retval None.  
  */
void DrvSPI_SetSlaveSelectActiveLevel(E_DRVSPI_PORT eSpiPort, E_DRVSPI_SS_ACT_TYPE eSSActType)
{
    SPI_PORT[eSpiPort]->SSR_BITS.SS_LVL = eSSActType;
}

/**
  * @brief  Get the level-trigger transmission status.
  * @param  eSpiPort: Specify the SPI port.  
  * @retval TRUE: The received trasaction number and received bits met the requirement which defines in TX_NUM
  *				  and TX_BIT_LEN among one transfer.				
  *			FALSE: The transaction number or the received bit length of one transaction doesn't meet the requirement
  *				  in one transfer.
  */
uint8_t DrvSPI_GetLevelTriggerStatus(E_DRVSPI_PORT eSpiPort)
{
#if (0)
	if(SPI_PORT[eSpiPort]->STATUS_BITS.LTRIG_FLAG==1)
		return TRUE;
	else
		return FALSE;
#endif	
	return FALSE;
}

/**
  * @brief  Enable the automatic slave select function and set the specified slave select pin.
  * @param  eSpiPort: Specify the SPI port.
  *			eSlaveSel: Select the slave select pins which will be used. 
  * @retval None.
  */
void DrvSPI_EnableAutoSS(E_DRVSPI_PORT eSpiPort, E_DRVSPI_SLAVE_SEL eSlaveSel)
{
    SPI_PORT[eSpiPort]->SSR_BITS.AUTOSS = 1;
    SPI_PORT[eSpiPort]->SSR_BITS.SSR = eSlaveSel;
}

/**
  * @brief  Disable the Automatic Slave Slect function and deselect slave select pins.
  * @param  eSpiPort: Specify the SPI port.  
  * @retval None.
  */
void DrvSPI_DisableAutoSS(E_DRVSPI_PORT eSpiPort)
{
    SPI_PORT[eSpiPort]->SSR_BITS.AUTOSS = 0;
    SPI_PORT[eSpiPort]->SSR_BITS.SSR = eDRVSPI_NONE;
}

/**
  * @brief  Configure the slave select pins.
  * @param  eSpiPort: Specify the SPI port.
  *			eSlaveSel: In automatic slave select operation, to use this parameter to select the slave select pins
  *					   which will be used. 
  *					   In manual slave select operation, the specified slave select pins will be set to active state.
  *					   It could be SPI_NONE, SPI_SS0, SPI_SS1 or SPI_SS0_SS1.
  * @retval None.
  */
void DrvSPI_SetSS(E_DRVSPI_PORT eSpiPort, E_DRVSPI_SLAVE_SEL eSlaveSel)
{
    SPI_PORT[eSpiPort]->SSR_BITS.SSR = eSlaveSel;
}

/**
  * @brief  Set the specified slave select pins to inactive state.
  * @param  eSpiPort: Specify the SPI port.  
  *			eSlaveSel: Specify slave select pins.
  *						It could be SPI_NONE, SPI_SS0, SPI_SS1 or SPI_SS0_SS1.
  * @retval None.
  */
void DrvSPI_ClrSS(E_DRVSPI_PORT eSpiPort, E_DRVSPI_SLAVE_SEL eSlaveSel)
{
	uint32_t u32Reg;
	
	u32Reg = SPI_PORT[eSpiPort]->SSR_BITS.SSR;
	u32Reg = u32Reg & (~eSlaveSel);
	SPI_PORT[eSpiPort]->SSR_BITS.SSR = u32Reg;
}

/**
  * @brief  Check the busy status of the specified SPI port.
  * @param  eSpiPort: Specify the SPI port.  
  * @retval TRUE:  The SPI port is in busy.
  *			FALSE: The SPI port is not in busy.
  */
uint8_t DrvSPI_IsBusy(E_DRVSPI_PORT eSpiPort)
{
    return ((SPI_PORT[eSpiPort]->CTL_BITS.GO_BUSY)?TRUE:FALSE);
}

/**
  * @brief  Configure the burst transfer settings. 
  * @param  eSpiPort: Specify the SPI port. 
  *			i32BurstCnt: Specify the transaction number in one transfer. It could be 1 or 2.
  *			i32Interval: Specify the delay clocks between successive transactions. It could be 2~17.
  * @retval SPI_ERR_BURST_CNT: The transaction number is out of range.
  *			SPI_ERR_TRANSMIT_INTERVAL: The suspend interval setting is out of range.
  *			E_SUCCESS: Success. 
  */
int32_t DrvSPI_BurstTransfer(E_DRVSPI_PORT eSpiPort, int32_t i32BurstCnt, int32_t i32Interval)
{
#if(0)
	if((i32BurstCnt < 1) || (i32BurstCnt > 2))
	{
		return E_DRVSPI_ERR_BURST_CNT;
	}
	
	if((i32Interval < 2) || (i32Interval > 17))
	{
		return E_DRVSPI_ERR_SUSPEND_INTERVAL;
	}

	SPI_PORT[eSpiPort]->CTL_BITS.TX_NUM = i32BurstCnt-1;
	SPI_PORT[eSpiPort]->CTL_BITS.SP_CYCLE = i32Interval-2;
	
#endif
	
	return E_SUCCESS;
}

/**
  * @brief  Configure the SPI clock.
  * @param  eSpiPort: Specify the SPI port.  
  *			u32Clock1: Specify the SPI clock rate in Hz. It's the clock rate of SPI base clock or variable clock 1.
  *			u32Clock2: Specify the SPI clock rate in Hz. It's the clock rate of variable clock 2.
  * @retval The actual value of divisor 1 is returned. SPI engine clock rate: APB clock rate / (divisor 1 * 2)
  *			The actual clock may be different to the target SPI clock due to hardware limitation.
  */
uint32_t DrvSPI_SetClockFreq(E_DRVSPI_PORT eSpiPort, uint32_t u32Clock1, uint32_t u32Clock2)
{
    uint32_t u32Div;
    uint32_t u32Pclk, u32CalculatedSpeed=0;
    
    u32Pclk = SystemCoreClock;//DrvSYS_GetHCLKFreq();
    
	if(u32Clock2!=0)
	{
		u32Div = (((u32Pclk / u32Clock2) + 1) >> 1) - 1;
		if(u32Div > 0xFFFF)
		    u32Div = 0xFFFF;
		
		u32CalculatedSpeed = u32Pclk / (2*(u32Div+1));
		if(u32CalculatedSpeed > u32Clock2)
			u32Div ++;
			
		
		SPI_PORT[eSpiPort]->CLKDIV_BITS.DIVIDER2 = u32Div;
	}
	else
		SPI_PORT[eSpiPort]->CLKDIV_BITS.DIVIDER2 = 0xFFFF;
	
	if(u32Clock1!=0)
	{
		u32Div = ((u32Pclk / u32Clock1) >> 1);
		if(u32Div > 0xFFFF)
		    u32Div = 0xFFFF;
		
		if(u32Div == 0)
			u32CalculatedSpeed = u32Pclk;
		else
			u32CalculatedSpeed = u32Pclk / (u32Div << 1);
		
		if(u32CalculatedSpeed > u32Clock1)
		{	
			u32Div ++;
			u32CalculatedSpeed = u32Pclk / (u32Div << 1);
		}
		
		SPI_PORT[eSpiPort]->CLKDIV_BITS.DIVIDER1 = u32Div;
	}
	else
	{
		u32Div = 0xFFFF;
		SPI_PORT[eSpiPort]->CLKDIV_BITS.DIVIDER1 = u32Div;
	}
	
	return u32CalculatedSpeed;
}

/**
  * @brief  Get the SPI engine clock rate in Hz.
  * @param  eSpiPort: Specify the SPI port.  
  * @retval The current SPI bus clock frequency in Hz.
  */
uint32_t DrvSPI_GetClock1Freq(E_DRVSPI_PORT eSpiPort)
{
	uint32_t u32Div;
	uint32_t u32ApbClock;

	u32ApbClock = SystemCoreClock;//DrvSYS_GetHCLKFreq();
	u32Div = SPI_PORT[eSpiPort]->CLKDIV_BITS.DIVIDER1;
	
	if(u32Div == 0)	return u32ApbClock;
	else return ((u32ApbClock >> 1) / u32Div);   /* SPI_CLK = APB_CLK / (Divider * 2) */
}


/**
  * @brief  Get the clock rate of variable clock 2 in Hz.
  * @param  eSpiPort: Specify the SPI port.  
  * @retval The frequency of variable clock 2 in Hz.
  */
uint32_t DrvSPI_GetClock2Freq(E_DRVSPI_PORT eSpiPort)
{
	uint32_t u32Div;
	uint32_t u32ApbClock;

	u32ApbClock = SystemCoreClock;//DrvSYS_GetHCLKFreq();
	u32Div = SPI_PORT[eSpiPort]->CLKDIV_BITS.DIVIDER2;
	return ((u32ApbClock >> 1) / (u32Div + 1));   /* SPI_CLK = APB_CLK / ((Divider + 1) * 2) */
}

/**
  * @brief  Set the variable clock pattern.
  * @param  eSpiPort: Specify the SPI port.
  *			bEnable: TRUE/FALSE -- Enable/Disable variable clock
  *         u32Pattern: Specify the variable clock pattern
  * @retval None.
  */
void DrvSPI_SetVariableClockFunction(E_DRVSPI_PORT eSpiPort, uint8_t bEnable, uint32_t u32Pattern)
{
	if(bEnable)
	{
		SPI_PORT[eSpiPort]->CTL_BITS.VARCLK_EN = 1;
		SPI_PORT[eSpiPort]->VARCLK = u32Pattern;
	}
	else
		SPI_PORT[eSpiPort]->CTL_BITS.VARCLK_EN = 0;
}

/**
  * @brief  Enable the SPI interrupt of the specified SPI port and install the callback function.
  * @param  eSpiPort: Specify the SPI port.
  *			pfnCallback: The callback function of SPI interrupt.
  *			u32UserData: The parameter which will be passed to the callback function.
  * @retval None.
  */
void DrvSPI_EnableInt(E_DRVSPI_PORT eSpiPort, PFN_DRVSPI_CALLBACK pfnCallback, uint32_t u32UserData)
{
	
	if(pfnCallback != NULL)
	{
		g_sSpiHandler[eSpiPort].pfncallback = pfnCallback;
		g_sSpiHandler[eSpiPort].u32userData = u32UserData;
	}
	
	SPI_PORT[eSpiPort]->CTL_BITS.INTEN = 1;
	
	if(eSpiPort == eDRVSPI_PORT0)
	{
	    NVIC_EnableIRQ(SPI0_IRQn);
	}
	else if(eSpiPort == eDRVSPI_PORT1)
	{
	    NVIC_EnableIRQ(SPI1_IRQn);
	}
	else if(eSpiPort == eDRVSPI_PORT2)
	{
	    NVIC_EnableIRQ(SPI2_IRQn);
	}
}	

/**
  * @brief  Disable the SPI interrupt.
  * @param  eSpiPort: Specify the SPI port.
  * @retval None.
  */
void DrvSPI_DisableInt(E_DRVSPI_PORT eSpiPort)
{
	g_sSpiHandler[eSpiPort].pfncallback = NULL;
	g_sSpiHandler[eSpiPort].u32userData = 0;
	
	SPI_PORT[eSpiPort]->CTL_BITS.INTEN = 0;

	if(eSpiPort == eDRVSPI_PORT0)
	{
	    NVIC_DisableIRQ(SPI0_IRQn);
	}
	else if(eSpiPort == eDRVSPI_PORT1)
	{
	    NVIC_DisableIRQ(SPI1_IRQn);
	}
	else if(eSpiPort == eDRVSPI_PORT2)
	{
	    NVIC_DisableIRQ(SPI2_IRQn);
	}	
}

/**
  * @brief  Return the SPI interrupt flag
  * @param  eSpiPort: Specify the SPI port.
  * @retval 0: SPI interrupt doesn't occur
  *         1: SPI interrupt occurs  
  */
uint32_t DrvSPI_GetIntFlag(E_DRVSPI_PORT eSpiPort)
{
	return (SPI_PORT[eSpiPort]->STATUS_BITS.INTSTS);
}

/**
  * @brief  Clear the SPI interrupt flag.
  * @param  eSpiPort: Specify the SPI port.
  * @retval None.
  */
void DrvSPI_ClrIntFlag(E_DRVSPI_PORT eSpiPort)
{
	SPI_PORT[eSpiPort]->STATUS_BITS.INTSTS = 1;
}

/**
  * @brief  Read data from SPI Rx registers and trigger SPI for next transfer.
  * @param  eSpiPort: Specify the SPI port.
  *			pu32Data:  Data got from the SPI bus.
  * @retval TRUE:  The data stored in pu32Data is valid.
  *			FALSE: The data stored in pu32Data is invalid.
  */
uint8_t DrvSPI_SingleRead(E_DRVSPI_PORT eSpiPort, uint32_t *pu32Data)
{
	if(SPI_PORT[eSpiPort]->CTL_BITS.GO_BUSY==1)
		return FALSE;

	*pu32Data = SPI_PORT[eSpiPort]->RX0;
	SPI_PORT[eSpiPort]->CTL_BITS.GO_BUSY = 1;
	return TRUE;
}

/**
  * @brief  Write data to SPI bus and trigger SPI to start transfer.
  * @param  eSpiPort: Specify the SPI port.
  *			pu32Data:  Data will be put to SPI bus.
  * @retval TRUE:  The data stored in pu32Data has been transferred.
  *			FALSE: The SPI is in busy. The data stored in pu32Data has not been transferred.
  */
uint8_t DrvSPI_SingleWrite(E_DRVSPI_PORT eSpiPort, uint32_t *pu32Data)
{
	if(SPI_PORT[eSpiPort]->CTL_BITS.GO_BUSY==1)
		return FALSE;

	SPI_PORT[eSpiPort]->TX0 = *pu32Data;
	SPI_PORT[eSpiPort]->CTL_BITS.GO_BUSY = 1;
	return TRUE;
}

/**
  * @brief  Read two words of data from SPI Rx registers and then trigger SPI for next transfer.
  * @param  eSpiPort: Specify the SPI port.
  *			pu32Buf:  Data got from the SPI bus.
  * @retval TRUE:  The data stored in pu32Buf is valid. 
  *			FALSE: The data stored in pu32Buf is invalid.
  */
uint8_t DrvSPI_BurstRead(E_DRVSPI_PORT eSpiPort, uint32_t *pu32Buf)
{
	if(SPI_PORT[eSpiPort]->CTL_BITS.GO_BUSY==1)
		return FALSE;

	pu32Buf[0] = SPI_PORT[eSpiPort]->RX0;
	pu32Buf[1] = SPI_PORT[eSpiPort]->RX1;
	SPI_PORT[eSpiPort]->CTL_BITS.GO_BUSY = 1; //????

	return TRUE;
}

/**
  * @brief  Write two words of data to SPI bus and then trigger SPI to start transfer.
  * @param  eSpiPort: Specify the SPI port.
  *			pu32Buf:  Store the data which will be transmitted through the SPI bus.
  * @retval TRUE:  The data stored in pu32Buf has been transferred. 
  *			FALSE: The SPI is in busy. The data stored in pu32Buf has not been transferred.
  */
uint8_t DrvSPI_BurstWrite(E_DRVSPI_PORT eSpiPort, uint32_t *pu32Buf)
{
	if(SPI_PORT[eSpiPort]->CTL_BITS.GO_BUSY==1)
		return FALSE;

	SPI_PORT[eSpiPort]->TX0 = pu32Buf[0];
	SPI_PORT[eSpiPort]->TX1 = pu32Buf[1];
	SPI_PORT[eSpiPort]->CTL_BITS.GO_BUSY = 1;

	return TRUE;
}

/**
  * @brief  Read data from Rx registers. This function will not trigger another data transfer.
  * @param  eSpiPort: Specify the SPI port.
  *			pu32Buf:  Store the data got from Rx registers.
  *			u32DataCount: The count of data read from Rx registers.
  * @retval The count of data actually read from Rx registers.
  */
uint32_t DrvSPI_DumpRxRegister(E_DRVSPI_PORT eSpiPort, uint32_t *pu32Buf, uint32_t u32DataCount)
{
	if(u32DataCount>2)
		u32DataCount = 2;
	
	pu32Buf[0] = SPI_PORT[eSpiPort]->RX0;
	
	if(u32DataCount == 2)
		pu32Buf[1] = SPI_PORT[eSpiPort]->RX1;
	
    return u32DataCount;
}

/**
  * @brief  Write data to Tx registers. This function will not trigger another data transfer.
  * @param  eSpiPort: Specify the SPI port.
  *			pu32Buf:  Store the data which will be written to Tx registers.
  *			u32DataCount: The count of data write to Tx registers.
  * @retval The count of data actually written to Tx registers.
  */
uint32_t DrvSPI_SetTxRegister(E_DRVSPI_PORT eSpiPort, uint32_t *pu32Buf, uint32_t u32DataCount)
{
	if(u32DataCount>2)
		u32DataCount = 2;
	
	SPI_PORT[eSpiPort]->TX0 = pu32Buf[0];
	
	if(u32DataCount == 2)
		SPI_PORT[eSpiPort]->TX1 = pu32Buf[1];
		
    return u32DataCount;
}

/**
  * @brief  Set the GO_BUSY bit to trigger a SPI data trasfer.
  * @param  eSpiPort: Specify the SPI port.
  * @retval None.
  */
void DrvSPI_SetGo(E_DRVSPI_PORT eSpiPort)
{
	SPI_PORT[eSpiPort]->CTL_BITS.GO_BUSY = 1;
}

/**
  * @brief  Clear the GO_BUSY bit.
  * @param  eSpiPort: Specify the SPI port.
  * @retval None.
  */
void DrvSPI_ClrGo(E_DRVSPI_PORT eSpiPort)
{
	SPI_PORT[eSpiPort]->CTL_BITS.GO_BUSY = 0;
}

/**
  * @brief  Configure the DMA settings.
  * @param  eSpiPort: Specify the SPI port.
  *			eDmaMode: Specify the DMA mode(SPI_TX_DMA/SPI_RX_DMA).
  *			bEnable: TRUE/FALSE  -- Enable/Disable DMA
  * @retval None.
  */
void DrvSPI_SetPDMA(E_DRVSPI_PORT eSpiPort, E_DRVSPI_DMA_MODE eDmaMode, uint8_t bEnable)
{
	if(eDmaMode==eDRVSPI_TX_DMA)
	{
		if(bEnable)
			SPI_PORT[eSpiPort]->PDMA_BITS.TX_DMA_EN = 1;
		else
			SPI_PORT[eSpiPort]->PDMA_BITS.TX_DMA_EN = 0;
	}
	else
	{
		if(bEnable)
			SPI_PORT[eSpiPort]->PDMA_BITS.RX_DMA_EN = 1;
		else
			SPI_PORT[eSpiPort]->PDMA_BITS.RX_DMA_EN = 0;
	}
}

/**
  * @brief  Configure 3-wire SPI function.
  * @param  eSpiPort: Specify the SPI port.
  *			bEnable: TRUE/FALSE  -- Enable/Disable 3-wire SPI function.
  * @retval None.
  */
void DrvSPI_3WireConfigure(E_DRVSPI_PORT eSpiPort, uint8_t bEnable)
{
	if(bEnable)
	{
		SPI_PORT[eSpiPort]->SSR_BITS.NOSLVSEL = 1;
		//enable slave start interrupt
		SPI_PORT[eSpiPort]->SSR_BITS.SSTA_INTEN = 1;
	}
	else
		SPI_PORT[eSpiPort]->SSR_BITS.NOSLVSEL = 0;	
}

/**
  * @brief  Abort transfer when using 3-wire SPI.
  *			If using 3-wire SPI as slave, when slave start interrupt status is set but 
  *			transfer done flag doesn't be set over a reasonable time, use this function
  *			to abort this transfer and restart the transfer.
  * @param  eSpiPort: Specify the SPI port.
  * @retval None.
  */
void DrvSPI_3WireAbort(E_DRVSPI_PORT eSpiPort)
{
	SPI_PORT[eSpiPort]->SSR_BITS.SLV_ABORT = 1;
}

/**
  * @brief  Check whether 3-wire SPI is started or not.
  * @param  eSpiPort: Specify the SPI port.
  * @retval TRUE: The transfer has been started.
  *			FALSE: THe transfer doesn't start yet.
  */
 
uint8_t DrvSPI_3WireCheckStartFlag(E_DRVSPI_PORT eSpiPort)
{
	return (SPI_PORT[eSpiPort]->STATUS_BITS.SLV_START_INTSTS == 1? TRUE:FALSE);
}

/**
  * @brief  Configure SPI wakeup function.
  * @param  eSpiPort: Specify the SPI port.
  *			bEnable: TRUE/FALSE  -- Enable/Disable SPI wakeup function.
  * @retval None.
  */
void DrvSPI_WakeupConfigure(E_DRVSPI_PORT eSpiPort, uint8_t bEnable)
{
	if(bEnable)
		SPI_PORT[eSpiPort]->CTL_BITS.WKEUP_EN = 1;
	else
		SPI_PORT[eSpiPort]->CTL_BITS.WKEUP_EN = 0;	
}

/**
  * @brief  Clear TX FIFO.
  * @param  eSpiPort: Specify the SPI port.
  * @retval None.
  */
void DrvSPI_ClearTxFIFO(E_DRVSPI_PORT eSpiPort)
{
// 	SPI_PORT[eSpiPort]->FFCLR_BITS.TX_CLR = 1;
}

/**
  * @brief  Clear RX FIFO.
  * @param  eSpiPort: Specify the SPI port.
  * @retval None.
  */
void DrvSPI_ClearRxFIFO(E_DRVSPI_PORT eSpiPort)
{
// 	SPI_PORT[eSpiPort]->FFCLR_BITS.RX_CLR = 1;
}

/*** (C) COPYRIGHT 2010 Nuvoton Technology Corp. ***/
